@@ -50,7 +50,7 @@ retry_static:
 			out_of_memory(0, cast_uchar "tsearch static", 0);
 			goto retry_static;
 		}
-		if ((unsigned char *)*p != &empty) internal("cache_add_to_tree: static entry not added: %p, %p", *p, &empty);
+		if ((unsigned char *)*p != &empty) internal_error("cache_add_to_tree: static entry not added: %p, %p", *p, &empty);
 	}
 #endif
 
@@ -60,7 +60,7 @@ retry:
 		out_of_memory(0, cast_uchar "tsearch", 0);
 		goto retry;
 	}
-	if ((unsigned char *)*p != e->url) internal("cache_add_to_tree: url '%s' is already present", e->url);
+	if ((unsigned char *)*p != e->url) internal_error("cache_add_to_tree: url '%s' is already present", e->url);
 }
 
 static void cache_delete_from_tree(struct cache_entry *e)
@@ -70,7 +70,7 @@ static void cache_delete_from_tree(struct cache_entry *e)
 	if (!e->url[0]) return;
 
 	p = tdelete(e->url, &cache_root, ce_compare);
-	if (!p) internal("cache_delete_from_tree: url '%s' not found", e->url);
+	if (!p) internal_error("cache_delete_from_tree: url '%s' not found", e->url);
 }
 
 static struct cache_entry *cache_search_tree(unsigned char *url)
@@ -117,7 +117,7 @@ my_uintptr_t cache_info(int type)
 			foreach(struct cache_entry, ce, lce, cache) i += is_entry_used(ce);
 			return i;
 		default:
-			internal("cache_info: bad request");
+			internal_error("cache_info: bad request");
 	}
 	return 0;
 }
@@ -137,7 +137,7 @@ my_uintptr_t decompress_info(int type)
 			foreach(struct cache_entry, ce, lce, cache) i += ce->decompressed && ce->refcount;
 			return i;
 		default:
-			internal("compress_info: bad request");
+			internal_error("compress_info: bad request");
 	}
 	return 0;
 }
@@ -179,8 +179,8 @@ int get_connection_cache_entry(struct connection *c)
 			add_to_str(&s, &l, a);
 		if (c->last_lookup_state.addr.n > 1) {
 			int i, d = 0;
-			if (l) add_to_str(&s, &l, cast_uchar " ");
-			add_to_str(&s, &l, cast_uchar "(");
+			if (l) add_chr_to_str(&s, &l, ' ');
+			add_chr_to_str(&s, &l, '(');
 			for (i = 0; i < c->last_lookup_state.addr.n; i++) {
 				if (i == c->last_lookup_state.addr_index)
 					continue;
@@ -192,7 +192,7 @@ int get_connection_cache_entry(struct connection *c)
 					d = 1;
 				}
 			}
-			add_to_str(&s, &l, cast_uchar ")");
+			add_chr_to_str(&s, &l, ')');
 		}
 		e->ip_address = s;
 	}
@@ -340,7 +340,7 @@ int defrag_entry(struct cache_entry *e)
 		g != &e->frag &&
 		list_struct(g, struct fragment)->offset <= list_struct(g->prev, struct fragment)->offset + list_struct(g->prev, struct fragment)->length; g = g->next)
 			if (list_struct(g, struct fragment)->offset < list_struct(g->prev, struct fragment)->offset + list_struct(g->prev, struct fragment)->length) {
-				internal("fragments overlay");
+				internal_error("fragments overlay");
 				return S_INTERNAL;
 			}
 	if (g == f->list_entry.next) {
@@ -460,9 +460,9 @@ void trim_cache_entry(struct cache_entry *e)
 
 void delete_cache_entry(struct cache_entry *e)
 {
-	if (e->refcount) internal("deleting locked cache entry");
+	if (e->refcount) internal_error("deleting locked cache entry");
 #ifdef DEBUG
-	if (is_entry_used(e)) internal("deleting loading cache entry");
+	if (is_entry_used(e)) internal_error("deleting loading cache entry");
 #endif
 	cache_delete_from_tree(e);
 	delete_entry_content(e);
@@ -472,6 +472,7 @@ void delete_cache_entry(struct cache_entry *e)
 	if (e->ip_address) mem_free(e->ip_address);
 #ifdef HAVE_SSL
 	if (e->ssl_info) mem_free(e->ssl_info);
+	if (e->ssl_authority) mem_free(e->ssl_authority);
 #endif
 	mem_free(e);
 }
@@ -488,7 +489,7 @@ static int shrink_file_cache(int u)
 	foreachback(struct cache_entry, e, le, cache) {
 		if (e->refcount || is_entry_used(e)) {
 			if (ncs < (my_uintptr_t)e->data_size) {
-				internal("cache_size underflow: %lu, %lu", (unsigned long)ncs, (unsigned long)e->data_size);
+				internal_error("cache_size underflow: %lu, %lu", (unsigned long)ncs, (unsigned long)e->data_size);
 			}
 			ncs -= (my_uintptr_t)e->data_size;
 		} else if (u == SH_FREE_SOMETHING) {
@@ -504,8 +505,8 @@ static int shrink_file_cache(int u)
 		ccs += (my_uintptr_t)e->data_size;
 		ccs2 += e->decompressed_len;
 	}
-	if (ccs != cache_size) internal("cache size badly computed: %lu != %lu", (unsigned long)cache_size, (unsigned long)ccs), cache_size = ccs;
-	if (ccs2 != decompressed_cache_size) internal("decompressed cache size badly computed: %lu != %lu", (unsigned long)decompressed_cache_size, (unsigned long)ccs2), decompressed_cache_size = ccs2;
+	if (ccs != cache_size) internal_error("cache size badly computed: %lu != %lu", (unsigned long)cache_size, (unsigned long)ccs), cache_size = ccs;
+	if (ccs2 != decompressed_cache_size) internal_error("decompressed cache size badly computed: %lu != %lu", (unsigned long)decompressed_cache_size, (unsigned long)ccs2), decompressed_cache_size = ccs2;
 	if (u == SH_CHECK_QUOTA && ncs <= (my_uintptr_t)memory_cache_size) goto ret;
 	foreachback(struct cache_entry, e, le, cache) {
 		if (u == SH_CHECK_QUOTA && (longlong)ncs <= (longlong)memory_cache_size * MEMORY_CACHE_GC_PERCENT) goto g;
@@ -515,11 +516,11 @@ static int shrink_file_cache(int u)
 		}
 		e->tgc = 1;
 		if (ncs < (my_uintptr_t)e->data_size) {
-			internal("cache_size underflow: %lu, %lu", (unsigned long)ncs, (unsigned long)e->data_size);
+			internal_error("cache_size underflow: %lu, %lu", (unsigned long)ncs, (unsigned long)e->data_size);
 		}
 		ncs -= (my_uintptr_t)e->data_size;
 	}
-	if (ncs) internal("cache_size(%lu) is larger than size of all objects(%lu)", (unsigned long)cache_size, (unsigned long)(cache_size - ncs));
+	if (ncs) internal_error("cache_size(%lu) is larger than size of all objects(%lu)", (unsigned long)cache_size, (unsigned long)(cache_size - ncs));
 	g:
 	if (le->next == &cache) goto ret;
 	le = le->next;
