@@ -219,7 +219,7 @@ void make_connection(struct connection *c, int port, int *sock, void (*func)(str
 		return;
 	}
 	if (c->newconn)
-		internal("already making a connection");
+		internal_error("already making a connection");
 	sl = strlen(cast_const_char host);
 	if (sl > MAXINT - sizeof(struct conn_info)) overalloc();
 	b = mem_calloc(sizeof(struct conn_info) + sl);
@@ -717,7 +717,7 @@ void continue_connection(struct connection *c, int *sock, void (*func)(struct co
 {
 	struct conn_info *b;
 	if (c->newconn)
-		internal("already making a connection");
+		internal_error("already making a connection");
 	b = mem_calloc(sizeof(struct conn_info));
 	b->func = func;
 	b->sock = sock;
@@ -942,12 +942,15 @@ static void write_select(void *c_)
 	struct write_buffer *wb;
 	int wr;
 	if (!(wb = c->buffer)) {
-		internal("write socket has no buffer");
+		internal_error("write socket has no buffer");
 		setcstate(c, S_INTERNAL);
 		abort_connection(c);
 		return;
 	}
-	set_connection_timeout(c);
+	if (wb->pos)
+		set_connection_timeout(c);
+	else
+		set_connection_timeout_keepal(c);
 	/*printf("ws: %d\n",wb->len-wb->pos);
 	for (wr = wb->pos; wr < wb->len; wr++) printf("%c", wb->data[wr]);
 	printf("-\n");*/
@@ -1024,7 +1027,7 @@ static void read_select(void *c_)
 	struct read_buffer *rb;
 	int rd;
 	if (!(rb = c->buffer)) {
-		internal("read socket has no buffer");
+		internal_error("read socket has no buffer");
 		setcstate(c, S_INTERNAL);
 		abort_connection(c);
 		return;
@@ -1115,7 +1118,7 @@ void read_from_socket(struct connection *c, int s, struct read_buffer *buf, void
 void kill_buffer_data(struct read_buffer *rb, int n)
 {
 	if (n > rb->len || n < 0) {
-		internal("called kill_buffer_data with bad value");
+		internal_error("called kill_buffer_data with bad value");
 		rb->len = 0;
 		return;
 	}
