@@ -7,6 +7,10 @@
 
 #include "links.h"
 
+#ifdef HAIKU
+#include <FindDirectory.h>
+#endif
+
 unsigned char system_name[MAX_STR_LEN];
 
 static void get_system_name(void)
@@ -28,13 +32,13 @@ static void get_system_name(void)
 			unsigned char *str = init_str();
 			int l = 0;
 			add_to_str(&str, &l, cast_uchar name.sysname);
-			add_to_str(&str, &l, cast_uchar " ");
+			add_chr_to_str(&str, &l, ' ');
 #ifdef OPENVMS
 			add_to_str(&str, &l, cast_uchar name.version);
 #else
 			add_to_str(&str, &l, cast_uchar name.release);
 #endif
-			add_to_str(&str, &l, cast_uchar " ");
+			add_chr_to_str(&str, &l, ' ');
 #ifdef OPENVMS
 			p = cast_uchar name.nodename + sizeof(name.nodename);
 			if ((unsigned char *)(&name + 1) - p >= 16 && memchr(cast_const_char p, 0, 16))
@@ -749,6 +753,13 @@ skip_path_conv:;
 			}
 		}
 	}
+#endif
+#ifdef HAIKU
+	home = mem_alloc(B_FILE_NAME_LENGTH);
+	if (find_directory(B_USER_SETTINGS_DIRECTORY, -1, 1, cast_char home, B_FILE_NAME_LENGTH) != B_OK) {
+		mem_free(home);
+		home = NULL;
+	}
 #endif
 	if (!home) home = stracpy(cast_uchar getenv("HOME"));
 #ifdef WIN
@@ -796,7 +807,9 @@ skip_path_conv:;
 		}
 	} else {
 		add_dot_links:
-#ifdef OPENVMS
+#if defined(DOS)
+		add_to_strn(&home_links, cast_uchar "links.cfg");
+#elif defined(OPENVMS) || defined(HAIKU)
 		add_to_strn(&home_links, cast_uchar "links");
 #else
 		add_to_strn(&home_links, cast_uchar ".links");
@@ -892,7 +905,7 @@ static int write_config_data(unsigned char *prefix, unsigned char *name, struct
 	}
 	add_to_strn(&config_file, name);
 	if ((err = write_to_config_file(config_file, c, 1))) {
-		if (term) msg_box(term, NULL, TEXT_(T_CONFIG_ERROR), AL_CENTER, TEXT_(T_UNABLE_TO_WRITE_TO_CONFIG_FILE), cast_uchar ": ", get_err_msg(err), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
+		if (term) msg_box(term, NULL, TEXT_(T_CONFIG_ERROR), AL_CENTER, TEXT_(T_UNABLE_TO_WRITE_TO_CONFIG_FILE), cast_uchar ": ", get_err_msg(err, term), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		mem_free(c);
 		mem_free(config_file);
 		return -1;
@@ -906,7 +919,7 @@ static void add_nm(struct option *o, unsigned char **s, int *l)
 {
 	if (*l) add_to_str(s, l, cast_uchar NEWLINE);
 	add_to_str(s, l, cast_uchar o->cfg_name);
-	add_to_str(s, l, cast_uchar " ");
+	add_chr_to_str(s, l, ' ');
 }
 
 static void add_quoted_to_str(unsigned char **s, int *l, unsigned char *q)
@@ -1129,13 +1142,13 @@ static void type_wr(struct option *o, unsigned char **s, int *l)
 		struct assoc *as = get_struct(a, struct assoc, head);
 		add_nm(o, s, l);
 		add_quoted_to_str(s, l, as->label);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_quoted_to_str(s, l, as->ct);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_quoted_to_str(s, l, as->prog);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_num_to_str(s, l, (!!as->cons) + (!!as->xwin) * 2 + (!!as->ask) * 4 + (!as->block) * 8 + (!!as->block) * 16 + (!!as->accept_http) * 32 + (!!as->accept_ftp) * 64);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_num_to_str(s, l, as->system);
 	}
 }
@@ -1163,7 +1176,7 @@ static void ext_wr(struct option *o, unsigned char **s, int *l)
 		struct extension *e = get_struct(a, struct extension, head);
 		add_nm(o, s, l);
 		add_quoted_to_str(s, l, e->ext);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_quoted_to_str(s, l, e->ct);
 	}
 }
@@ -1194,7 +1207,7 @@ static void prog_wr(struct option *o, unsigned char **s, int *l)
 		if (!*a->prog) continue;
 		add_nm(o, s, l);
 		add_quoted_to_str(s, l, a->prog);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_num_to_str(s, l, a->system);
 	}
 }
@@ -1303,23 +1316,23 @@ static void term_wr(struct option *o, unsigned char **s, int *l)
 	foreachback(struct term_spec, ts, lts, term_specs) {
 		add_nm(o, s, l);
 		add_quoted_to_str(s, l, ts->term);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_num_to_str(s, l, ts->mode);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_num_to_str(s, l, !!ts->m11_hack + !!ts->braille * 2);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_num_to_str(s, l, !!ts->col + !!ts->restrict_852 * 2 + !!ts->block_cursor * 4);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		if (ts->character_set == -1) add_to_str(s, l, cast_uchar "default");
 		else add_to_str(s, l, get_cp_mime_name(ts->character_set));
 		if (ts->left_margin || ts->right_margin || ts->top_margin || ts->bottom_margin) {
-			add_to_str(s, l, cast_uchar " ");
+			add_chr_to_str(s, l, ' ');
 			add_num_to_str(s, l, ts->left_margin);
-			add_to_str(s, l, cast_uchar " ");
+			add_chr_to_str(s, l, ' ');
 			add_num_to_str(s, l, ts->right_margin);
-			add_to_str(s, l, cast_uchar " ");
+			add_chr_to_str(s, l, ' ');
 			add_num_to_str(s, l, ts->top_margin);
-			add_to_str(s, l, cast_uchar " ");
+			add_chr_to_str(s, l, ' ');
 			add_num_to_str(s, l, ts->bottom_margin);
 		}
 	}
@@ -1392,14 +1405,14 @@ static void dp_wr(struct option *o, unsigned char **s, int *l)
 		if (dp->nosave) continue;
 		add_nm(o, s, l);
 		add_quoted_to_str(s, l, dp->name);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_quoted_to_str(s, l, dp->param ? dp->param : (unsigned char*)"");
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_quoted_to_str(s, l, dp->shell_term);
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		if (dp->kbd_codepage == -1) add_to_str(s, l, cast_uchar "default");
 		else add_to_str(s, l, get_cp_mime_name(dp->kbd_codepage));
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_num_to_str(s, l, dp->palette_mode);
 		/* pokud se sem neco prida, opravit podminku na zacatku cyklu */
 	}
@@ -1641,7 +1654,7 @@ fprintf(stdout, "%s%s%s%s%s%s\n",
 "    (default: 600)\n"
 "\n"
 " -timeout-when-trying-multiple-addresses <sec>\n"
-"  Timeout for connection when trying multiple addresses.\n"
+"  Timeout for connection when trying multiple addresses or keepalive connection.\n"
 "    (default: 3)\n"
 "\n"
 " -bind-address <ip address>\n"
@@ -1883,6 +1896,34 @@ fprintf(stdout, "%s%s%s%s%s%s\n",
 "    (valid for svgalib and framebuffer).\n"
 "    Overwriting may or may not be faster, depending on hardware.\n"
 "\n"
+#ifdef HAVE_FREETYPE
+" -font <file>\n"
+"  Font file for normal text\n"
+"\n"
+" -font-bold <file>\n"
+"  Font file for bold text\n"
+"\n"
+" -font-monospaced <file>\n"
+"  Font file for monospaced text\n"
+"\n"
+" -font-monospaced-bold <file>\n"
+"  Font file for monospaced bold text\n"
+"\n"
+#ifdef USE_ITALIC
+" -font-italic <file>\n"
+"  Font file for normal text\n"
+"\n"
+" -font-italic-bold <file>\n"
+"  Font file for bold text\n"
+"\n"
+" -font-monospaced-italic <file>\n"
+"  Font file for monospaced text\n"
+"\n"
+" -font-monospaced-italic-bold <file>\n"
+"  Font file for monospaced bold text\n"
+"\n"
+#endif
+#endif
 #ifdef JS
 " -enable-javascript <0>/<1>\n"
 "  Enable javascript.\n"
@@ -2125,6 +2166,18 @@ int dither_images = 1;
 int gamma_bits = 2;	/*0 --- 8, 1 --- 16, 2 --- auto */
 int overwrite_instead_of_scroll = 1;
 
+unsigned char font_file[MAX_STR_LEN] = "";
+unsigned char font_file_b[MAX_STR_LEN] = "";
+unsigned char font_file_m[MAX_STR_LEN] = "";
+unsigned char font_file_m_b[MAX_STR_LEN] = "";
+
+#ifdef USE_ITALIC
+unsigned char font_file_i[MAX_STR_LEN] = "";
+unsigned char font_file_i_b[MAX_STR_LEN] = "";
+unsigned char font_file_i_m[MAX_STR_LEN] = "";
+unsigned char font_file_i_m_b[MAX_STR_LEN] = "";
+#endif
+
 
 int menu_font_size = G_BFU_DEFAULT_FONT_SIZE;
 
@@ -2261,6 +2314,16 @@ static struct option links_options[] = {
 	{1, gen_cmd, num_rd, num_wr, 0, 1, &dither_images, "dither_images", "dither-images"},
 	{1, gen_cmd, num_rd, num_wr, 0, 2, &gamma_bits, "gamma_correction", "gamma-correction"},
 	{1, gen_cmd, num_rd, num_wr, 0, 1, &overwrite_instead_of_scroll, "overwrite_instead_of_scroll", "overwrite-instead-of-scroll"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file, "font", "font"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file_b, "font_bold", "font-bold"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file_m, "font_monospaced", "font-monospaced"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file_m_b, "font_monospaced_bold", "font-monospaced-bold"},
+#ifdef USE_ITALIC
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file_i, "font_italic", "font-italic"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file_i_b, "font_italic_bold", "font-italic-bold"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file_i_m, "font_monospaced_italic", "font-monospaced-italic"},
+	{1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, font_file_i_m_b, "font_monospaced_italic_bold", "font-monospaced-italic-bold"},
+#endif
 	{1, gen_cmd, num_rd, NULL, 0, 1, &js_enable, "enable_javascript", NULL},
 	{1, gen_cmd, num_rd, NULL, 0, 1, &js_verbose_errors, "verbose_javascript_errors", NULL},
 	{1, gen_cmd, num_rd, NULL, 0, 1, &js_verbose_warnings, "verbose_javascript_warnings", NULL},
