@@ -192,7 +192,7 @@ int slow_fpu = -1;
 		int r,g,b,o,rt,gt,bt,y,x;\
 		unsigned char *my_restrict outp=out->data;\
 		int *my_restrict bptr;\
-		int skip=out->skip-SKIP_CODE;\
+		ssize_t skip = out->skip - SKIP_CODE;\
 \
 		o=0;o=o; /*warning go away */\
 		switch(out->x){\
@@ -228,7 +228,7 @@ int slow_fpu = -1;
 		unsigned short ir, ig, ib;\
 		int rt,gt,bt,o,x,y;\
 		unsigned char *my_restrict outp=out->data;\
-		int skip=out->skip-SKIP_CODE;\
+		ssize_t skip = out->skip - SKIP_CODE;\
 \
 		o=0;o=o; /*warning go away */\
 		for (y=out->y;y;y--){\
@@ -909,7 +909,7 @@ static void make_real_colors_table(void)
 	if (!real_colors)
 		return;
 	real_colors_table = mem_alloc(256 * 3 * sizeof(unsigned short));
-	agx_48_to_48(real_colors_table, real_colors, 256, display_red_gamma tcc_hack, display_green_gamma tcc_hack, display_blue_gamma tcc_hack);
+	agx_48_to_48(real_colors_table, real_colors, 256, (float)(display_red_gamma tcc_hack), (float)(display_green_gamma tcc_hack), (float)(display_blue_gamma tcc_hack));
 	mem_free(real_colors);
 }
 
@@ -1090,7 +1090,7 @@ void init_dither(int depth)
 		break;
 
 		default:
-		internal("Graphics driver returned unsupported pixel memory organisation %d",depth);
+		internal_error("Graphics driver returned unsupported pixel memory organisation %d",depth);
 	}
 
 	make_round_tables();
