@@ -509,7 +509,7 @@ static int do_queued_lookup(struct dnsquery *q)
 #ifndef THREAD_SAFE_LOOKUP
 	} else {
 		/*debug("queuing lookup for %s", q->name);*/
-		if (dns_queue->next_in_queue) internal("DNS queue corrupted");
+		if (dns_queue->next_in_queue) internal_error("DNS queue corrupted");
 		dns_queue->next_in_queue = q;
 		dns_queue = q;
 		return 1;
@@ -688,7 +688,7 @@ unsigned long dns_info(int type)
 		case CI_FILES:
 			return list_size(&dns_cache);
 		default:
-			internal("dns_info: bad request");
+			internal_error("dns_info: bad request");
 	}
 	return 0;
 }
