@@ -314,28 +314,47 @@ void check_memory_leaks(void)
 {
 #if defined(LEAK_DEBUG) && !defined(NO_IE)
 	if (mem_amount || mem_blocks) {
+		char rb[OS_REPORT_ERROR_BUFFER];
+		char *rp = rb;
+
 		fatal_tty_exit();
+
 		fprintf(stderr, "\n"ANSI_SET_BOLD"Memory leak by %lu bytes (%lu blocks)"ANSI_CLEAR_BOLD"\n", (unsigned long)mem_amount, (unsigned long)mem_blocks);
+		snprintf(rp, sizeof rb - (rp - rb), "Memory leak by %lu bytes (%lu blocks)", (unsigned long)mem_amount, (unsigned long)mem_blocks);
+		rp = strchr(rp, 0);
+
 #ifdef LEAK_DEBUG_LIST
 		fprintf(stderr, "\nList of blocks: ");
+		snprintf(rp, sizeof rb - (rp - rb), ":\r\n");
+		rp = strchr(rp, 0);
+
 		{
 			int r = 0;
 			struct alloc_header *ah;
 			struct list_head *lah;
 			foreach(struct alloc_header, ah, lah, memory_list) {
-				fprintf(stderr, "%s%p:%lu @ %s:%d", r ? ", ": "", (unsigned char *)ah + L_D_S, (unsigned long)ah->size, ah->file, ah->line), r = 1;
-				if (ah->comment) fprintf(stderr, ":\"%s\"", ah->comment);
+				fprintf(stderr, "%s%p:%lu @ %s:%d", r ? ", ": "", (unsigned char *)ah + L_D_S, (unsigned long)ah->size, ah->file, ah->line);
+				snprintf(rp, sizeof rb - (rp - rb), "%s%p:%lu @ %s:%d", r ? ", ": "", (unsigned char *)ah + L_D_S, (unsigned long)ah->size, ah->file, ah->line);
+				rp = strchr(rp, 0);
+
+				if (ah->comment) {
+					fprintf(stderr, ":\"%s\"", ah->comment);
+					snprintf(rb, sizeof rb - (rp - rb), ":\"%s\"", ah->comment);
+					rp = strchr(rp, 0);
+				}
+				r = 1;
 			}
 			fprintf(stderr, "\n");
 		}
 #endif
+		os_report_error("Memory leak", rb, NULL);
 		force_dump();
 	}
 #endif
 	exit_heap();
 }
 
-static void er(int b, char *m, va_list l)
+static void er(int b, const char *m, va_list l)
 {
 #ifdef HAVE_VPRINTF
 	vfprintf(stderr, cast_const_char m, l);
@@ -348,19 +367,30 @@ static void er(int b, char *m, va_list l)
 	portable_sleep(1000);
 }
 
-void error(char *m, ...)
+void error(const char *m, ...)
 {
 	va_list l;
+
+	va_start(l, m);
+	os_report_error_va("Error", m, l);
+	va_end(l);
+
 	va_start(l, m);
 	fprintf(stderr, "\n");
 	er(1, m, l);
 	va_end(l);
 }
 
-void fatal_exit(char *m, ...)
+void fatal_exit(const char *m, ...)
 {
 	va_list l;
+
+	va_start(l, m);
+	os_report_error_va("Fatal error", m, l);
+	va_end(l);
+
 	fatal_tty_exit();
+
 	va_start(l, m);
 	fprintf(stderr, "\n");
 	er(1, m, l);
@@ -375,23 +405,31 @@ unsigned char *errfile;
 
 static unsigned char errbuf[4096];
 
-void int_error(char *m, ...)
+void int_error(const char *m, ...)
 {
 #ifdef NO_IE
 	return;
 #else
 	va_list l;
-	fatal_tty_exit();
+
+	sprintf(cast_char errbuf, "Internal error at %s:%d\r\n%s", errfile, errline, m);
 	va_start(l, m);
+	os_report_error_va("Internal error", cast_const_char errbuf, l);
+	va_end(l);
+
+	fatal_tty_exit();
+
 	sprintf(cast_char errbuf, "\n"ANSI_SET_BOLD"INTERNAL ERROR"ANSI_CLEAR_BOLD" at %s:%d: %s", errfile, errline, m);
+	va_start(l, m);
 	er(1, cast_char errbuf, l);
 	va_end(l);
+
 	force_dump();
 	exit(RET_INTERNAL);
 #endif
 }
 
-void debug_msg(char *m, ...)
+void debug_msg(const char *m, ...)
 {
 	va_list l;
 	va_start(l, m);
