@@ -312,7 +312,7 @@ static void roman(unsigned char *p, unsigned n)
 		}
 		i++;
 		if (n && !roman_tbl[i].n) {
-			internal("BUG in roman number convertor");
+			internal_error("BUG in roman number convertor");
 			return;
 		}
 	}
@@ -531,11 +531,11 @@ static unsigned char *get_target(unsigned char *a)
 void kill_html_stack_item(struct html_element *e)
 {
 	if (!e || (void *)e == &html_stack) {
-		internal("trying to free bad html element");
+		internal_error("trying to free bad html element");
 		return;
 	}
 	if (e->dontkill == 2) {
-		internal("trying to kill unkillable element");
+		internal_error("trying to kill unkillable element");
 		return;
 	}
 	html_format_changed = 1;
@@ -897,7 +897,7 @@ static unsigned char *find_element_end(unsigned char *a)
 	for (p = a - 1; *p != '<'; p--)
 		;
 	if (parse_element(p, eoff, NULL, NULL, NULL, &p)) {
-		internal("parse element failed");
+		internal_error("parse element failed");
 		return a;
 	}
 	return p;
@@ -1105,17 +1105,18 @@ static void html_img(unsigned char *a)
 	ismap = format_.link && (F || !has_attr(a, cast_uchar "usemap")) && has_attr(a, cast_uchar "ismap");
 	if (format_.image) mem_free(format_.image), format_.image = NULL;
 	if (
-		(s = get_url_val_img(a, cast_uchar "data-full")) ||
-		(s = get_url_val_img(a, cast_uchar "data-src")) ||
-		(s = get_url_val_img(a, cast_uchar "data-normal")) ||
 		(s = get_url_val_img(a, cast_uchar "data-defer-src")) ||
-		(s = get_url_val_img(a, cast_uchar "data-li-src")) ||
-		(s = get_url_val_img(a, cast_uchar "data-original")) ||
-		(s = get_url_val_img(a, cast_uchar "data-small")) ||
+		(s = get_url_val_img(a, cast_uchar "data-delay_url")) ||
+		(s = get_url_val_img(a, cast_uchar "data-full")) ||
 		(s = get_url_val_img(a, cast_uchar "data-lazy")) ||
 		(s = get_url_val_img(a, cast_uchar "data-lazy-src")) ||
-		(s = get_url_val_img(a, cast_uchar "data-delay_url")) ||
+		(s = get_url_val_img(a, cast_uchar "data-li-src")) ||
+		(s = get_url_val_img(a, cast_uchar "data-normal")) ||
+		(s = get_url_val_img(a, cast_uchar "data-original")) ||
+		(s = get_url_val_img(a, cast_uchar "data-small")) ||
 		(s = get_url_val_img(a, cast_uchar "data-source")) ||
+		(s = get_url_val_img(a, cast_uchar "data-src")) ||
+		(s = get_url_val_img(a, cast_uchar "data-thumb")) ||
 		(s = get_url_val_img(a, cast_uchar "src")) ||
 		(s = get_url_val_img(a, cast_uchar "dynsrc")) ||
 		(s = get_url_val_img(a, cast_uchar "data")) ||
@@ -1734,7 +1735,7 @@ static void find_form_for_input(unsigned char *i)
 	}
 	if (last_form_tag && last_input_tag && i > last_input_tag) {
 		if (parse_element(last_form_tag, i, &name, &namelen, &la, &s))
-			internal("couldn't parse already parsed tag");
+			internal_error("couldn't parse already parsed tag");
 		lf = last_form_tag;
 		s = last_input_tag;
 	} else {
@@ -1842,7 +1843,7 @@ static void set_max_textarea_width(int *w)
 		}
 #ifdef G
 	} else {
-		struct style *st = g_get_style(0, 0, d_opt->font_size, cast_uchar(G_HTML_DEFAULT_FAMILY "-medium-roman-serif-mono"), 0);
+		struct style *st = g_get_style(0, 0, d_opt->font_size, FF_MONOSPACED);
 		int uw = g_char_width(st, '_');
 		g_free_style(st);
 		if (uw && *w > limit / uw) {
@@ -1872,7 +1873,7 @@ static void html_input(unsigned char *a)
 	else if (!casestrcmp(al, cast_uchar "radio")) fc->type = FC_RADIO;
 	else if (!casestrcmp(al, cast_uchar "submit")) fc->type = FC_SUBMIT;
 	else if (!casestrcmp(al, cast_uchar "reset")) fc->type = FC_RESET;
-	else if (!casestrcmp(al, cast_uchar "file")) fc->type = FC_FILE;
+	else if (!casestrcmp(al, cast_uchar "file")) fc->type = FC_FILE_UPLOAD;
 	else if (!casestrcmp(al, cast_uchar "hidden")) fc->type = FC_HIDDEN;
 	else if (!casestrcmp(al, cast_uchar "image")) fc->type = FC_IMAGE;
 	else if (!casestrcmp(al, cast_uchar "button")) fc->type = FC_BUTTON;
@@ -1889,9 +1890,9 @@ static void html_input(unsigned char *a)
 	fc->target = stracpy(form.target);
 	fc->name = get_attr_val(a, cast_uchar "name");
 	if (fc->type == FC_TEXT || fc->type == FC_PASSWORD) fc->default_value = get_attr_val(a, cast_uchar "value");
-	else if (fc->type != FC_FILE) fc->default_value = get_exact_attr_val(a, cast_uchar "value");
+	else if (fc->type != FC_FILE_UPLOAD) fc->default_value = get_exact_attr_val(a, cast_uchar "value");
 	if (fc->type == FC_CHECKBOX && !fc->default_value) fc->default_value = stracpy(cast_uchar "on");
-	if ((size = get_num(a, cast_uchar "size")) <= 0) size = HTML_DEFAULT_INPUT_SIZE;
+	if ((size = get_num(a, cast_uchar "size")) <= 1) size = HTML_DEFAULT_INPUT_SIZE;
 	size++;
 	if (size > HTML_MINIMAL_TEXTAREA_WIDTH) {
 		set_max_textarea_width(&size);
@@ -1916,7 +1917,7 @@ static void html_input(unsigned char *a)
 	switch (fc->type) {
 		case FC_TEXT:
 		case FC_PASSWORD:
-		case FC_FILE:
+		case FC_FILE_UPLOAD:
 			format_.attr |= AT_BOLD | AT_FIXED;
 			format_.fontsize = 3;
 			for (i = 0; i < fc->size; i++) put_chrs(cast_uchar "_", 1);
@@ -1962,7 +1963,7 @@ static void html_input(unsigned char *a)
 			put_chrs(cast_uchar "&nbsp;]", 7);
 			break;
 		default:
-			internal("bad control type");
+			internal_error("bad control type");
 	}
 	kill_html_stack_item(&html_top);
 	put_chrs(cast_uchar " ", 1);
@@ -2342,7 +2343,7 @@ static int do_html_select(unsigned char *attr, unsigned char *html, unsigned cha
 
 static void html_textarea(unsigned char *a)
 {
-	internal("This should be never called");
+	internal_error("This should be never called");
 }
 
 static void do_html_textarea(unsigned char *attr, unsigned char *html, unsigned char *eof, unsigned char **end)
@@ -2556,14 +2557,14 @@ static void parse_frame_widths(unsigned char *a, int ww, int www, int **op, int
 		}
 		if (q < 0) {
 			q = 0;
-			/*internal("parse_frame_widths: q < 0"); may happen when page contains too big values */
+			/*internal_error("parse_frame_widths: q < 0"); may happen when page contains too big values */
 		}
 		for (i = 0; i < ol; i++) if (oo[i] < 0) {
 			if (q) o[i]++, q--;
 		}
 		if (q > 0) {
 			q = 0;
-			/*internal("parse_frame_widths: q > 0"); may happen when page contains too big values */
+			/*internal_error("parse_frame_widths: q > 0"); may happen when page contains too big values */
 		}
 		mem_free(oo);
 	}
@@ -2654,7 +2655,9 @@ static void html_meta(unsigned char *a)
 		if (!strcmp(cast_const_char prop, "og:image")) {
 			unsigned char *host = get_host_name(format_.href_base);
 			if (host) {
-				if (casestrstr(host, cast_uchar "flickr.") ||
+				if (casestrstr(host, cast_uchar "facebook.") ||
+				    casestrstr(host, cast_uchar "flickr.") ||
+				    casestrstr(host, cast_uchar "imgur.") ||
 				    casestrstr(host, cast_uchar "instagram.") ||
 				    casestrstr(host, cast_uchar "mastodon.") ||
 				    casestrstr(host, cast_uchar "pinterest.") ||
@@ -2725,10 +2728,17 @@ static void html_link(unsigned char *a)
 	}
 	if (!casestrcmp(name, cast_uchar "dns-prefetch")) {
 		unsigned char *pre_url, *host;
+		if (dmp || *proxies.socks_proxy || proxies.only_proxies) {
+			goto skip;
+		}
 		pre_url = join_urls(format_.href_base, url);
+		if (get_proxy_string(pre_url) || is_noproxy_url(pre_url)) {
+			mem_free(pre_url);
+			goto skip;
+		}
 		host = get_host_name(pre_url);
 		mem_free(pre_url);
-		if (!dmp) dns_prefetch(host);
+		dns_prefetch(host);
 		mem_free(host);
 		goto skip;
 	}
@@ -2828,7 +2838,7 @@ static struct element_info elements[] = {
 	{"BUTTON",	html_button,	0, 0},
 
 	{"META",	html_meta,	0, 1},
-	{"LINK",	html_link,	1, 1},
+	{"LINK",	html_link,	0, 1},
 	{"IFRAME",	html_iframe,	1, 1},
 	{"FRAME",	html_frame,	1, 1},
 	{"FRAMESET",	html_frameset,	1, 0},
@@ -2881,7 +2891,7 @@ static int qd(unsigned char *html, unsigned char *eof, int *len)
 	int l;
 	*len = 1;
 	if (html >= eof) {
-		internal("qd: out of data, html == %p, eof == %p", html, eof);
+		internal_error("qd: out of data, html == %p, eof == %p", html, eof);
 		return -1;
 	}
 	if (html[0] != '&' || d_opt->plain & 1) return html[0];
