@@ -164,7 +164,7 @@ struct table {
 
 
 #ifdef DEBUG
-#define CELL(t, x, y) (((x) < 0 || (x) >= (t)->rx || (y) < 0 || (y) >= (t)->ry) ? (internal("accessing cell out of table (%d,%d) - limit (%d,%d)", (x), (y), (t)->rx, (t)->ry), (t)->cells) : &(t)->cells[(y) * (t)->rx + (x)])
+#define CELL(t, x, y) (((x) < 0 || (x) >= (t)->rx || (y) < 0 || (y) >= (t)->ry) ? (internal_error("accessing cell out of table (%d,%d) - limit (%d,%d)", (x), (y), (t)->rx, (t)->ry), (t)->cells) : &(t)->cells[(y) * (t)->rx + (x)])
 #else
 #define CELL(t, x, y) (&(t)->cells[(y) * (t)->rx + (x)])
 #endif
@@ -612,7 +612,7 @@ static struct table *parse_table(unsigned char *html, unsigned char *eof, unsign
 			if (!(sc = new_cell(t, x + k, y + j)) || sc->used) {
 				int l, m;
 				if (sc->mx == x && sc->my == y) continue;
-				/*internal("boo");*/
+				/*internal_error("boo");*/
 				for (l = 0; l < k; l++) memset(CELL(t, x + l, y + j), 0, sizeof(struct table_cell));
 				rsp = j;
 				for (l = 0; l < i; l++) for (m = 0; m < j; m++) CELL(t, x + l, y + m)->rowspan = j;
@@ -671,7 +671,7 @@ static void get_cell_width(struct table *t, struct table_cell *c, int w, int a,
 #endif
 	}
 	/*debug("get_cell_width: %d < %d", *min, *max);*/
-	/*if (min && max && *min > *max) internal("get_cell_width: %d > %d", *min, *max);*/
+	/*if (min && max && *min > *max) internal_error("get_cell_width: %d > %d", *min, *max);*/
 }
 
 #define g_c_w(cc)							\
@@ -708,7 +708,7 @@ static void dst_width(int *p, int n, int w, int *lim)
 		if (lim && p[i] > lim[i]) w = safe_add(w, p[i] - lim[i]), p[i] = lim[i];
 	}
 	if (w) {
-		/*if (!lim) internal("bug in dst_width");*/
+		/*if (!lim) internal_error("bug in dst_width");*/
 		lim = NULL;
 		s = 0;
 		goto again;
@@ -824,7 +824,7 @@ static int get_column_widths(struct table *t)
 			struct table_cell *c = CELL(t, i, j);
 			if (c->spanned || !c->used) continue;
 			if (c->colspan + i > t->x) {
-				/*internal("colspan out of table");
+				/*internal_error("colspan out of table");
 				return -1;*/
 				continue;
 			}
@@ -882,7 +882,7 @@ static void get_table_width(struct table *t)
 	}
 	t->min_t = min;
 	t->max_t = max;
-	/*if (min > max) internal("min(%d) > max(%d)", min, max);*/
+	/*if (min > max) internal_error("min(%d) > max(%d)", min, max);*/
 }
 
 static void distribute_widths(struct table *t, int width)
@@ -896,7 +896,7 @@ static void distribute_widths(struct table *t, int width)
 	t->rw = 0;
 	if (!t->x) return;
 	if (d < 0) {
-		/*internal("too small width %d, required %d", width, t->min_t);*/
+		/*internal_error("too small width %d, required %d", width, t->min_t);*/
 		return;
 	}
 	for (i = 0; i < t->x; i++) if (t->max_c[i] > mmax_c) mmax_c = t->max_c[i];
@@ -949,7 +949,7 @@ static void distribute_widths(struct table *t, int width)
 					w[i] = 1, mx[i] = MAXINT;
 					break;
 				default:
-					/*internal("could not expand table");*/
+					/*internal_error("could not expand table");*/
 					goto end2;
 			}
 			p = safe_add(p, w[i]);
@@ -977,7 +977,7 @@ static void distribute_widths(struct table *t, int width)
 			d -= t->w_c[mii] - q;
 			while (d < 0) t->w_c[mii]--, d++;
 			if (t->w_c[mii] < q) {
-				/*internal("shrinking cell");*/
+				/*internal_error("shrinking cell");*/
 				t->w_c[mii] = q;
 			}
 			wq = 1;
@@ -1009,7 +1009,7 @@ static void check_table_widths(struct table *t)
 		if (c->x_width > p) {
 			/*int min, max;
 			get_cell_width(t, c, 0, 0, &min, &max, NULL);
-			internal("cell is now wider (%d > %d) min = %d, max = %d, now_min = %d, now_max = %d", c->x_width, p, t->min_c[i], t->max_c[i], min, max);*/
+			internal_error("cell is now wider (%d > %d) min = %d, max = %d, now_min = %d, now_max = %d", c->x_width, p, t->min_c[i], t->max_c[i], min, max);*/
 			/* sbohem, internale. chytl jsi mi spoustu chyb v tabulkovaci, ale ted je proste cas jit ... ;-( */
 			c->x_width = p;
 		}
@@ -1021,7 +1021,7 @@ static void check_table_widths(struct table *t)
 			struct table_cell *c = CELL(t, i, j);
 			if (!c->start) continue;
 			if (c->colspan + i > t->x) {
-				/*internal("colspan out of table");*/
+				/*internal_error("colspan out of table");*/
 				mem_free(w);
 				return;
 			}
@@ -1036,7 +1036,7 @@ static void check_table_widths(struct table *t)
 					c = 0;
 					for (l = i; l < i + s; l++) if (w[l] < t->w_c[k]) w[l] = safe_add(w[l], 1), w[k]--, c = 1;
 					if (w[k] > t->w_c[k]) {
-						if (!c) internal("can't shrink cell");
+						if (!c) internal_error("can't shrink cell");
 						else goto ag;
 					}
 				}*/
@@ -1054,7 +1054,7 @@ static void check_table_widths(struct table *t)
 		}*/
 	}
 	if (ns > s) {
-		/*internal("new width(%d) is larger than previous(%d)", ns, s);*/
+		/*internal_error("new width(%d) is larger than previous(%d)", ns, s);*/
 		mem_free(w);
 		return;
 	}
@@ -1114,7 +1114,7 @@ static void get_table_heights(struct table *t)
 #ifdef G
 			} else {
 				if (!(gp = g_format_html_part(cell->start, cell->end, cell->align, 0, xw, NULL, cell->link_num, NULL /* FIX: background image */, cell->bgcolor_str, t->gp->data))) return;
-				/*if (gp->root->xw > xw) internal("vono to neumi formatovat... buuuu (%d > %d)", gp->root->xw, xw);*/
+				/*if (gp->root->xw > xw) internal_error("vono to neumi formatovat... buuuu (%d > %d)", gp->root->xw, xw);*/
 				cell->root = gp->root;
 				gp->root = NULL;
 				cell->height = gp->y;
@@ -1561,7 +1561,7 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	nn->y = safe_add(p->yp, cye);
 	nn->xw = n->xw;
 	add_to_list(p->data->nodes, nn);
-	/*if (p->cy + t->rh != cye) internal("size does not match; 1:%d, 2:%d", p->cy + t->rh, cye);*/
+	/*if (p->cy + t->rh != cye) internal_error("size does not match; 1:%d, 2:%d", p->cy + t->rh, cye);*/
 	p->cy = cye;
 
 	ret2:
@@ -1601,7 +1601,7 @@ static void add_to_cell_sets(struct table_cell ****s, int **nn, int *n, struct r
 {
 	int i, j;
 	if (r->y1 < 0 || r->y2 < 0)
-		internal("add_to_cell_sets: integer overflow: %d, %d", r->y1, r->y2);
+		internal_error("add_to_cell_sets: integer overflow: %d, %d", r->y1, r->y2);
 	for (i = r->y1 >> RECT_BOUND_BITS; i <= (r->y2 - 1) >> RECT_BOUND_BITS; i++) {
 		if (i >= *n) {
 			struct table_cell ***ns;
