@@ -21,7 +21,7 @@ struct http_connection_info {
 static void http_send_header(struct connection *c);
 static void http_get_header(struct connection *c);
 static void test_restart(struct connection *c);
-static void add_user_agent(unsigned char **hdr, int *l);
+static void add_user_agent(unsigned char **hdr, int *l, unsigned char *url);
 static void add_referer(unsigned char **hdr, int *l, unsigned char *url, unsigned char *prev_url);
 static void add_accept(unsigned char **hdr, int *l);
 static void add_accept_language(unsigned char **hdr, int *l, struct http_connection_info *info);
@@ -222,7 +222,7 @@ static void http_send_header(struct connection *c)
 
 	proxy = is_proxy_url(c->url);
 	host = remove_proxy_prefix(c->url);
-	set_connection_timeout(c);
+	set_connection_timeout_keepal(c);
 	info = mem_calloc(sizeof(struct http_connection_info));
 	c->info = info;
 #ifdef HAVE_SSL
@@ -269,7 +269,7 @@ static void http_send_header(struct connection *c)
 		c->unrestartable = 2;
 	}
 	if (!proxy) {
-		add_to_str(&hdr, &l, cast_uchar "/");
+		add_chr_to_str(&hdr, &l, '/');
 		u = get_url_data(host);
 	} else {
 		u = host;
@@ -310,13 +310,19 @@ static void http_send_header(struct connection *c)
 		add_to_str(&hdr, &l, h);
 		mem_free(h);
 		if ((h = get_port_str(host))) {
-			add_chr_to_str(&hdr, &l, ':');
-			add_to_str(&hdr, &l, h);
+			unsigned char *default_port = cast_uchar "80";
+#ifdef HAVE_SSL
+			if (c->ssl) default_port = cast_uchar "443";
+#endif
+			if (strcmp(cast_const_char h, cast_const_char default_port)) {
+				add_chr_to_str(&hdr, &l, ':');
+				add_to_str(&hdr, &l, h);
+			}
 			mem_free(h);
 		}
 		add_to_str(&hdr, &l, cast_uchar "\r\n");
 	}
-	add_user_agent(&hdr, &l);
+	add_user_agent(&hdr, &l, info->https_forward ? NULL : host);
 	if (proxy) add_proxy_auth_string(&hdr, &l, c->url);
 	if (!info->https_forward) {
 		test_restart(c);
@@ -375,12 +381,28 @@ static void test_restart(struct connection *c)
 	}
 }
 
-static void add_user_agent(unsigned char **hdr, int *l)
+static void add_user_agent(unsigned char **hdr, int *l, unsigned char *url)
 {
 	add_to_str(hdr, l, cast_uchar "User-Agent: ");
 	if (SCRUB_HEADERS) {
 		add_to_str(hdr, l, cast_uchar "Mozilla/5.0 (Windows NT 6.1; rv:60.0) Gecko/20100101 Firefox/60.0\r\n");
 	} else if (!(*http_options.header.fake_useragent)) {
+
+		/*
+		 * Google started to return css-styled page for searches.
+		 * It returns non-css page if the user agent begins with Lynx.
+		 */
+		if (url &&
+			(casestrstr(url, cast_uchar "/www.google.") ||
+			 casestrstr(url, cast_uchar "/google.")) &&
+			strstr(cast_const_char url, "/search?") &&
+			(strstr(cast_const_char url, "?q=") ||
+			 strstr(cast_const_char url, "&q=")) &&
+			!strstr(cast_const_char url, "?tbm=isch") &&
+			!strstr(cast_const_char url, "&tbm=isch")
+			)
+				add_to_str(hdr, l, cast_uchar("Lynx/"));
+
 		add_to_str(hdr, l, cast_uchar("Links (" VERSION_STRING "; "));
 		add_to_str(hdr, l, system_name);
 		add_to_str(hdr, l, cast_uchar "; ");
@@ -485,7 +507,7 @@ static void add_accept_language(unsigned char **hdr, int *l, struct http_connect
 			int la;
 			la = *l;
 			add_to_str(hdr, l, get_text_translation(TEXT_(T__ACCEPT_LANGUAGE), NULL));
-			add_to_str(hdr, l, cast_uchar ",");
+			add_chr_to_str(hdr, l, ',');
 			if (!strstr(cast_const_char(*hdr + la), "en,") &&
 			    !strstr(cast_const_char(*hdr + la), "en;")) add_to_str(hdr, l, cast_uchar "en;q=0.2,");
 			add_to_str(hdr, l, cast_uchar "*;q=0.1\r\n");
@@ -536,6 +558,12 @@ static void add_accept_encoding(unsigned char **hdr, int *l, unsigned char *url,
 			add_to_str(hdr, l, cast_uchar "br");
 		}
 #endif
+#if defined(HAVE_ZSTD)
+		if (!SCRUB_HEADERS && !(info->bl_flags & BL_NO_BZIP2)) {
+			if (*l != l1) add_to_str(hdr, l, cast_uchar ", ");
+			add_to_str(hdr, l, cast_uchar "zstd");
+		}
+#endif
 #if defined(HAVE_BZIP2)
 		if (!SCRUB_HEADERS && !(info->bl_flags & BL_NO_BZIP2)) {
 			if (*l != l1) add_to_str(hdr, l, cast_uchar ", ");
@@ -577,7 +605,7 @@ static void add_accept_charset(unsigned char **hdr, int *l, struct http_connecti
 		int aclen = 0;
 		ac = init_str();
 		for (i = 0; (cs = get_cp_mime_name(i)); i++) {
-			if (aclen) add_to_str(&ac, &aclen, cast_uchar ",");
+			if (aclen) add_chr_to_str(&ac, &aclen, ',');
 			else add_to_str(&ac, &aclen, cast_uchar "Accept-Charset: ");
 			add_to_str(&ac, &aclen, cs);
 		}
@@ -1056,6 +1084,8 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 	if (c->ssl) {
 		if (e->ssl_info) mem_free(e->ssl_info);
 		e->ssl_info = get_cipher_string(c->ssl);
+		if (e->ssl_authority) mem_free(e->ssl_authority);
+		e->ssl_authority = stracpy(c->ssl->ca);
 	}
 #endif
 	if (e->redirect) mem_free(e->redirect), e->redirect = NULL;
@@ -1186,7 +1216,7 @@ static void http_got_header(struct connection *c, struct read_buffer *rb)
 static void http_get_header(struct connection *c)
 {
 	struct read_buffer *rb;
-	set_connection_timeout(c);
+	set_connection_timeout_keepal(c);
 	if (!(rb = alloc_read_buffer(c))) return;
 	rb->close = 1;
 	read_from_socket(c, c->sock1, rb, http_got_header);
