@@ -149,7 +149,7 @@ static void ssl_load_private_certificates(SSL_CTX *ctx)
 		int len = certificates[i].len;
 		unsigned char *b64;
 		if (data[len])
-			internal("invalid builtin certificate %u", i);
+			internal_error("invalid builtin certificate %u", i);
 #if 1
 		b64 = base64_encode(data, len, cast_uchar "-----BEGIN CERTIFICATE-----\n", cast_uchar "-----END CERTIFICATE-----", 6);
 		bio = BIO_new_mem_buf(b64, (int)strlen(cast_const_char b64));
@@ -340,6 +340,7 @@ links_ssl *getSSL(void)
 	ssl->bytes_read = ssl->bytes_written = 0;
 	ssl->session_set = 0;
 	ssl->session_retrieved = 0;
+	ssl->ca = NULL;
 	return ssl;
 }
 
@@ -357,6 +358,7 @@ void freeSSL(links_ssl *ssl)
 	}
 #endif
 	SSL_free(ssl->ssl);
+	if (ssl->ca) mem_free(ssl->ca);
 	mem_free(ssl);
 }
 
@@ -580,11 +582,44 @@ static int verify_ssl_host_name(X509 *server_cert, unsigned char *host)
 
 #endif
 
+static unsigned char *extract_field(unsigned char *str, unsigned char *field)
+{
+	size_t len;
+	unsigned char *f = cast_uchar strstr(cast_const_char str, cast_const_char field);
+	if (!f)
+		return NULL;
+	f += strlen(cast_const_char field);
+	len = strcspn(cast_const_char f, "/");
+	return memacpy(f, len);
+}
+
+static unsigned char *extract_ca(unsigned char *str)
+{
+	unsigned char *c, *o;
+	c = extract_field(str, cast_uchar "/C=");
+	o = extract_field(str, cast_uchar "/O=");
+	if (!o)
+		o = extract_field(str, cast_uchar "/CN=");
+	if (!o) {
+		if (c) mem_free(c), c = NULL;
+		o = stracpy(str);
+	}
+	if (c) {
+		add_to_strn(&o, cast_uchar ", ");
+		add_to_strn(&o, c);
+		mem_free(c);
+	}
+	return o;
+}
+
 int verify_ssl_certificate(links_ssl *ssl, unsigned char *host)
 {
 	X509 *server_cert;
 	int ret;
 
+	if (ssl->ca != NULL)
+		mem_free(ssl->ca), ssl->ca = NULL;
+
 	if (SSL_get_verify_result(ssl->ssl) != X509_V_OK) {
 		clear_ssl_errors(__LINE__);
 		return S_INVALID_CERTIFICATE;
@@ -595,6 +630,47 @@ int verify_ssl_certificate(links_ssl *ssl, unsigned char *host)
 		return S_INVALID_CERTIFICATE;
 	}
 	ret = verify_ssl_host_name(server_cert, host);
+	if (!ret) {
+#ifdef SSL_GET0_VERIFIED_CHAIN
+		STACK_OF(X509) *certs = SSL_get0_verified_chain(ssl->ssl);
+#else
+		STACK_OF(X509) *certs = SSL_get_peer_cert_chain(ssl->ssl);
+#endif
+		if (certs) {
+			int num = sk_X509_num(certs);
+			int i;
+			unsigned char *last_ca = NULL;
+			unsigned char *cas = init_str();
+			int casl = 0;
+			for (i = num - 1; i >= 0; i--) {
+				unsigned char space[3072];
+				unsigned char *n;
+				X509 *cert = sk_X509_value(certs, i);
+				X509_NAME *name;
+				name = X509_get_issuer_name(cert);
+				n = cast_uchar X509_NAME_oneline(name, cast_char space, 3072);
+				if (n) {
+					unsigned char *ca = extract_ca(n);
+					if (!last_ca || strcmp(cast_const_char ca, cast_const_char last_ca)) {
+						if (casl)
+							add_to_str(&cas, &casl, CERT_RIGHT_ARROW);
+						add_to_str(&cas, &casl, ca);
+						if (last_ca)
+							mem_free(last_ca);
+						last_ca = ca;
+					} else {
+						mem_free(ca);
+					}
+				}
+			}
+			if (last_ca)
+				mem_free(last_ca);
+			if (casl)
+				ssl->ca = cas;
+			else
+				mem_free(cas);
+		}
+	}
 	X509_free(server_cert);
 	clear_ssl_errors(__LINE__);
 	return ret;
@@ -794,7 +870,7 @@ unsigned long session_info(int type)
 		case CI_FILES:
 			return list_size(&session_cache);
 		default:
-			internal("session_info: bad request");
+			internal_error("session_info: bad request");
 	}
 	return 0;
 }
