@@ -22,11 +22,14 @@ static struct g_object_image *global_goi;
 struct cached_image *global_cimg;
 int end_callback_hit;
 
-static int is_image_size_sane(int x, int y)
+static int is_image_size_sane(ssize_t x, ssize_t y)
 {
-	unsigned a = (unsigned)x * (unsigned)y * 6;
-	if (y && a / (unsigned)y / 6 != (unsigned)x) return 0;
-	return a < MAXINT;
+	size_t a;
+	if (x < 0 || y < 0) return 0;
+	if (x >= MAXINT || y >= MAXINT) return 0;
+	a = (size_t)x * (size_t)y * (drv->depth & 7);
+	if (y && (size_t)a / (size_t)y / (drv->depth & 7) != (size_t)x) return 0;
+	return a <= MAX_SIZE_T / 2;
 }
 
 /* mem_free(cimg->decoder) */
@@ -122,7 +125,7 @@ void img_destruct_cached_image(struct cached_image *cimg)
 #ifdef DEBUG
 		default:
 		fprintf(stderr, "img_destruct_cached_image: state=%d\n", cimg->state);
-		internal("Invalid state in struct cached_image");
+		internal_error("Invalid state in struct cached_image");
 #endif /* #ifdef DEBUG */
 	}
 	mem_free(cimg->url);
@@ -139,48 +142,50 @@ void img_destruct_cached_image(struct cached_image *cimg)
  * Input may be 0. In this case output=0.
  * If input is >0 the output is also >0.
  */
-static int img_scale_h(unsigned scale, int in){
-	int out;
+static ssize_t img_scale_h(unsigned scale, ssize_t in)
+{
+	ssize_t out;
 	/* We assume unsigned long holds at least 32 bits */
 	unsigned long pre;
 
 	if (in<=0) return in;
 	pre=((unsigned long)(aspect<65536UL?65536UL:aspect)*scale+128)>>8;
-	out=(int)(((unsigned long)in*pre+12800UL)/25600UL);
+	out=(size_t)(((unsigned long)in*pre+12800UL)/25600UL);
 	if (out<1) out=1;
 	return out;
 }
 
-static int img_scale_v(unsigned scale, int in){
-	int out;
+static ssize_t img_scale_v(unsigned scale, ssize_t in)
+{
+	ssize_t out;
 	unsigned long divisor;
 
 	if (in<=0) return in;
 	divisor=(100*(aspect>=65536UL?65536UL:aspect)+128)>>8;
-	out=(int)(((unsigned long)in*(scale*256)+(divisor>>1))/divisor);
+	out=(ssize_t)(((unsigned long)in*(scale*256)+(divisor>>1))/divisor);
 	if (out<1) out=1;
 	return out;
 }
 
 /* Returns height (pixels) for prescribed width (pixels). Honours aspect. */
-static int width2height(double width_px, double width_mm, double height_mm)
+static ssize_t width2height(double width_px, double width_mm, double height_mm)
 {
-	int height_px;
+	ssize_t height_px;
 
 	if (width_px<=0) return 0;
-	height_px=(int)((height_mm*width_px*65536)/(aspect*width_mm));
+	height_px=(ssize_t)((height_mm*width_px*65536)/(aspect*width_mm));
 	if (height_px<1) height_px=1;
 	return height_px;
 
 }
 
 /* Returns width (pixels) for prescribed height (pixels). Honours aspect. */
-static int height2width(double height_px, double width_mm, double height_mm)
+static ssize_t height2width(double height_px, double width_mm, double height_mm)
 {
-	int width_px;
+	ssize_t width_px;
 
 	if (height_px<=0) return 0;
-	width_px=(int)((width_mm*height_px*aspect)/(65536*height_mm));
+	width_px=(ssize_t)((width_mm*height_px*aspect)/(65536*height_mm));
 	if (width_px<1) width_px=1;
 	return width_px;
 
@@ -228,7 +233,7 @@ int header_dimensions_known(struct cached_image *cimg)
 #ifdef DEBUG
 	if ((cimg->state^8)&13){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in header_dimensions_known");
+		internal_error("Invalid state in header_dimensions_known");
 	}
 #endif /* #ifdef DEBUG */
 	if (cimg->width<1||cimg->height<1){
@@ -295,33 +300,33 @@ int header_dimensions_known(struct cached_image *cimg)
 	if (cimg->strip_optimized){
 		struct bitmap tmpbmp;
 		unsigned short *buf_16;
-		int i;
+		ssize_t i;
 
-		tmpbmp.x=cimg->width;
-		tmpbmp.y=1;
+		tmpbmp.x = (int)cimg->width;
+		tmpbmp.y = 1;
 		/* No buffer, bitmap is valid from the very beginning */
-		cimg->bmp.x=cimg->width;
-		cimg->bmp.y=cimg->height;
+		cimg->bmp.x = (int)cimg->width;
+		cimg->bmp.y = (int)cimg->height;
 		if (drv->get_empty_bitmap(&(cimg->bmp))) {
 			cimg->dregs = NULL;
 			goto skip_img;
 		}
-		if ((unsigned)cimg->width > MAXINT / sizeof(*buf_16) / 3) overalloc();
-		buf_16=mem_alloc(sizeof(*buf_16)*3*cimg->width);
+		if ((size_t)cimg->width > MAX_SIZE_T / sizeof(*buf_16) / 3) overalloc();
+		buf_16 = mem_alloc(sizeof(*buf_16) * 3 * (size_t)cimg->width);
 		round_color_sRGB_to_48(&red, &green, &blue
 			, cimg->background_color);
 		mix_one_color_48(buf_16,cimg->width, red, green, blue);
 #ifdef DEBUG
 		if (cimg->height<=0){
-			fprintf(stderr,"cimg->height=%d\n",cimg->height);
-			internal("Invalid cimg->height in strip_optimized section of\
- header_dimensions_known");
+			fprintf(stderr,"cimg->height=%ld\n", (long)cimg->height);
+			internal_error("Invalid cimg->height in strip_optimized section of header_dimensions_known");
 		}
 #endif /* #ifdef DEBUG */
 		/* The skip is uninitialized here and is read by dither_start
 		 * but is not used in any malicious way so it doesn't matter
 		 */
-		tmpbmp.data=cimg->bmp.data;
+		tmpbmp.data = cimg->bmp.data;
+		tmpbmp.skip = cimg->bmp.skip;
 		cimg->dregs=dither_images?dither_start(buf_16,&tmpbmp):NULL;
 		tmpbmp.data=(unsigned char *)tmpbmp.data+cimg->bmp.skip;
 		if (cimg->dregs)
@@ -347,9 +352,8 @@ int header_dimensions_known(struct cached_image *cimg)
 	}else {
 		cimg->rows_added=1;
 		cimg->bmp_used=0;
-		if (cimg->width && (unsigned)cimg->width * (unsigned)cimg->height / (unsigned)cimg->width != (unsigned)cimg->height) overalloc();
-		if ((unsigned)cimg->width * (unsigned)cimg->height > (unsigned)MAXINT / cimg->buffer_bytes_per_pixel) overalloc();
-		cimg->buffer=mem_alloc_mayfail((size_t)cimg->width * (size_t)cimg->height * (size_t)cimg->buffer_bytes_per_pixel);
+		if (cimg->width && (size_t)cimg->width * (size_t)cimg->height / (size_t)cimg->width != (size_t)cimg->height) overalloc();
+		cimg->buffer = mem_alloc_mayfail((size_t)cimg->width * (size_t)cimg->height * (size_t)cimg->buffer_bytes_per_pixel);
 		if (!cimg->buffer)
 			return 1;
 		if (cimg->buffer_bytes_per_pixel==4
@@ -391,14 +395,14 @@ int header_dimensions_known(struct cached_image *cimg)
 /* Fills "tmp" buffer with the resulting data and does not free the input
  * buffer. May be called only in states 12 and 14 of cimg
  */
-static unsigned short *buffer_to_16(unsigned short *tmp, struct cached_image *cimg, unsigned char *buffer, int height)
+static unsigned short *buffer_to_16(unsigned short *tmp, struct cached_image *cimg, unsigned char *buffer, ssize_t height)
 {
 	unsigned short red, green, blue;
 
 #ifdef DEBUG
 	if (cimg->state != 12 && cimg->state != 14){
 		fprintf(stderr, "cimg->state=%d\n", cimg->state);
-		internal("invalid state in buffer_to_16");
+		internal_error("invalid state in buffer_to_16");
 	}
 #endif /* #ifdef DEBUG */
 	switch (cimg->buffer_bytes_per_pixel) {
@@ -443,7 +447,7 @@ static unsigned short *buffer_to_16(unsigned short *tmp, struct cached_image *ci
 			if (cimg->gamma_table) {
 				agx_and_uc_64_to_48_table(tmp, (unsigned short *)buffer, cimg->width * height, cimg->gamma_table, red, green, blue);
 			} else {
-				agx_and_uc_64_to_48(tmp ,(unsigned short*)buffer,cimg->width * height,
+				agx_and_uc_64_to_48(tmp, (unsigned short *)buffer, cimg->width * height,
 					(float)((float)(user_gamma tcc_hack) / cimg->red_gamma),
 					(float)((float)(user_gamma tcc_hack) / cimg->green_gamma),
 					(float)((float)(user_gamma tcc_hack) / cimg->blue_gamma),
@@ -453,7 +457,7 @@ static unsigned short *buffer_to_16(unsigned short *tmp, struct cached_image *ci
 
 #ifdef DEBUG
 		default:
-			internal("buffer_to_16: unknown mem organization");
+			internal_error("buffer_to_16: unknown mem organization");
 #endif /* #ifdef DEBUG */
 
 	}
@@ -475,57 +479,55 @@ static unsigned short *buffer_to_16(unsigned short *tmp, struct cached_image *ci
  * commited.
  * height must be >=1 !!!
  */
-void buffer_to_bitmap_incremental(struct cached_image *cimg
-	,unsigned char *buffer, int height, int yoff, int *dregs, int use_strip)
+void buffer_to_bitmap_incremental(struct cached_image *cimg,
+	unsigned char *buffer, ssize_t height, ssize_t yoff, int *dregs, int use_strip)
 {
 #define max_height 16
 /* max_height must be at least 1 */
 	unsigned short *tmp;
 	struct bitmap tmpbmp;
-	int add1=0, add2;
+	ssize_t add1 = 0, add2;
 
 #ifdef DEBUG
 	if (cimg->state!=12&&cimg->state!=14){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in buffer_to_bitmap_incremental\n");
+		internal_error("Invalid state in buffer_to_bitmap_incremental\n");
 	}
 	if (height<1){
-		fprintf(stderr,"height=%d\n",height);
-		internal("Invalid height in buffer_to_bitmap_incremental\n");
+		fprintf(stderr,"height=%ld\n", (long)height);
+		internal_error("Invalid height in buffer_to_bitmap_incremental\n");
 	}
 	if (cimg->width<1||cimg->height<1){
-		fprintf(stderr,"cimg->width=%d, cimg->height=%d\n",cimg->width,
-				cimg->height);
-		internal("Invalid cimg->width x cimg->height in\
+		fprintf(stderr,"cimg->width=%ld, cimg->height=%ld\n", (long)cimg->width, (long)cimg->height);
+		internal_error("Invalid cimg->width x cimg->height in\
 buffer_to_bitmap_incremental");
 	}
 #endif /* #ifdef DEBUG */
-	if ((unsigned)cimg->width > MAXINT / max_height / 3 / sizeof(*tmp)) overalloc();
-	tmp=mem_alloc(cimg->width*(height<max_height?height:max_height)*3*sizeof(*tmp));
+	if ((size_t)cimg->width > MAX_SIZE_T / (size_t)max_height / 3 / sizeof(*tmp)) overalloc();
+	tmp=mem_alloc((size_t)cimg->width * (size_t)(height < max_height ? height : max_height) * 3 * sizeof(*tmp));
 	/* Prepare a fake bitmap for dithering */
-	tmpbmp.x=cimg->width;
+	tmpbmp.x = (int)cimg->width;
 	if (!use_strip){
 	       tmpbmp.data=(unsigned char *)cimg->bmp.data+cimg->bmp.skip*yoff;
 	       add1=cimg->bmp.skip*max_height;
 	}
 	add2=cimg->buffer_bytes_per_pixel*cimg->width*max_height;
 not_enough:
-	tmpbmp.y=height<max_height?height:max_height;
+	tmpbmp.y = (int)(height < max_height ? height : max_height);
 	if (use_strip) {
-		tmpbmp.data=drv->prepare_strip(&(cimg->bmp),yoff,tmpbmp.y);
+		tmpbmp.data = drv->prepare_strip(&(cimg->bmp), (int)yoff, tmpbmp.y);
 		if (!tmpbmp.data) goto prepare_failed;
 	}
 	tmpbmp.skip=cimg->bmp.skip;
 	buffer_to_16(tmp, cimg, buffer, tmpbmp.y);
-	if (dregs){
+	if (dregs) {
 		dither_restart(tmp, &tmpbmp, dregs);
-	}
-	else {
+	} else {
 		(*round_fn)(tmp, &tmpbmp);
 	}
 	if (use_strip) {
 		prepare_failed:
-		drv->commit_strip(&(cimg->bmp),yoff,tmpbmp.y);
+		drv->commit_strip(&(cimg->bmp), (int)yoff, tmpbmp.y);
 	}
 	height-=tmpbmp.y;
 	if (!height) goto end;
@@ -552,20 +554,19 @@ end:
 static void buffer_to_bitmap(struct cached_image *cimg)
 {
 	unsigned short *tmp, *tmp1;
-	int ix, iy, ox, oy, gonna_be_smart;
+	ssize_t ix, iy, ox, oy;
+	int gonna_be_smart;
 	int *dregs;
 
 #ifdef DEBUG
 	if(cimg->state!=12&&cimg->state!=14){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("buffer_to_bitmap called in invalid state");
+		internal_error("buffer_to_bitmap called in invalid state");
 	}
-	if (cimg->strip_optimized) internal("strip_optimized in buffer_to_bitmap");
+	if (cimg->strip_optimized) internal_error("strip_optimized in buffer_to_bitmap");
 	if (cimg->width<1||cimg->height<1){
-		fprintf(stderr,"cimg->width=%d, cimg->height=%d\n",cimg->width,
-				cimg->height);
-		internal("Invalid cimg->width x cimg->height in\
-buffer_to_bitmap");
+		fprintf(stderr,"cimg->width=%ld, cimg->height=%ld\n", (long)cimg->width, (long)cimg->height);
+		internal_error("Invalid cimg->width x cimg->height in buffer_to_bitmap");
 	}
 #endif /* #ifdef DEBUG */
 
@@ -581,10 +582,10 @@ buffer_to_bitmap");
 	if (ix==ox&&iy==oy) gonna_be_smart=1;
 	else{
 		gonna_be_smart=0;
-		if (ix && (unsigned)ix * (unsigned)iy / (unsigned)ix != (unsigned)iy) overalloc();
-		if ((unsigned)ix * (unsigned)iy > MAXINT / sizeof(*tmp) / 3) overalloc();
-		tmp=mem_alloc_mayfail(ix*iy*3*sizeof(*tmp));
-		if (tmp) buffer_to_16(tmp,cimg,cimg->buffer,iy);
+		if (ix && (size_t)ix * (size_t)iy / (size_t)ix != (size_t)iy) overalloc();
+		if ((size_t)ix * (size_t)iy > MAX_SIZE_T / sizeof(*tmp) / 3) overalloc();
+		tmp = mem_alloc_mayfail((size_t)ix * (size_t)iy * 3 * sizeof(*tmp));
+		if (tmp) buffer_to_16(tmp, cimg, cimg->buffer, iy);
 		if (!cimg->decoder){
 			mem_free_buffer(cimg);
 			cimg->buffer=NULL;
@@ -593,7 +594,7 @@ buffer_to_bitmap");
 		/* Scale the image to said size */
 #ifdef DEBUG
 		if (ox<=0||oy<=0){
-			internal("ox or oy <=0 before resampling image");
+			internal_error("ox or oy <=0 before resampling image");
 		}
 #endif /* #ifdef DEBUG */
 		if (tmp && (ix!=ox || iy!=oy)) {
@@ -603,8 +604,8 @@ buffer_to_bitmap");
 		}
 	}
 	if (cimg->bmp_used) drv->unregister_bitmap(&cimg->bmp);
-	cimg->bmp.x=ox;
-	cimg->bmp.y=oy;
+	cimg->bmp.x = (int)ox;
+	cimg->bmp.y = (int)oy;
 	if (drv->get_empty_bitmap(&(cimg->bmp))) {
 		if (!gonna_be_smart) {
 			if (tmp) {
@@ -615,8 +616,8 @@ buffer_to_bitmap");
 	}
 	if (gonna_be_smart){
 		if (dither_images) {
-			if ((unsigned)cimg->width > MAXINT / 3 / sizeof(*dregs)) overalloc();
-			dregs = mem_calloc(sizeof(*dregs)*3*cimg->width);
+			if ((size_t)cimg->width > MAX_SIZE_T / 3 / sizeof(*dregs)) overalloc();
+			dregs = mem_calloc(sizeof(*dregs) * 3 * (size_t)cimg->width);
 		} else {
 			dregs = NULL;
 		}
@@ -679,7 +680,7 @@ void img_end(struct cached_image *cimg)
 #ifdef DEBUG
 		default:
 		fprintf(stderr,"state=%d\n",cimg->state);
-		internal("Invalid state encountered in end");
+		internal_error("Invalid state encountered in end");
 #endif /* #ifdef DEBUG */
 	}
 	cimg->state|=1;
@@ -736,7 +737,7 @@ static void r3l0ad(struct cached_image *cimg, struct g_object_image *goi)
 #ifdef DEBUG
 		default:
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state in r3l0ad()");
+		internal_error("Invalid state in r3l0ad()");
 #endif /* #ifdef DEBUG */
 	}
 	cimg->state&=2;
@@ -756,7 +757,7 @@ static void type(struct cached_image *cimg, unsigned char *content_type, unsigne
 #ifdef DEBUG
 	if (cimg->state!=0&&cimg->state!=2){
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid state encountered in type()");
+		internal_error("Invalid state encountered in type()");
 	}
 #endif /* #ifdef DEBUG */
 #ifdef HAVE_JPEG
@@ -844,10 +845,10 @@ static int img_process_download(struct g_object_image *goi, struct f_data_c *fda
 	int chopped=0;
 
 #ifdef DEBUG
-	if (!goi->af) internal("NULL goi->af in process_download\n");
+	if (!goi->af) internal_error("NULL goi->af in process_download\n");
 	if (cimg->state>=16){ /* Negative don't occur because it's unsigned char */
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid cimg->state in img_process_download\n");
+		internal_error("Invalid cimg->state in img_process_download\n");
 	}
 #endif /* #ifdef DEBUG */
 	if (!goi->af->rq) return 0;
@@ -918,7 +919,7 @@ static int img_process_download(struct g_object_image *goi, struct f_data_c *fda
 #ifdef DEBUG
 		default:
 			fprintf(stderr,"cimg->image_type=%d\n",cimg->state);
-			internal("Invalid image_type encountered when processing data in\
+			internal_error("Invalid image_type encountered when processing data in\
 img_process_download.\n");
 #endif /* #ifdef DEBUG */
 		}
@@ -949,8 +950,18 @@ img_process_download.\n");
 			img_end(cimg);
 		}
 	} else if (!chopped) {
-		if (fdatac && f_is_finished(fdatac->f_data)) {
-			refresh_image(fdatac, &goi->goti.go, 2000);
+		if (fdatac) {
+			if (f_is_finished(fdatac->f_data)) {
+				refresh_image(fdatac, &goi->goti.go, 2000);
+			} else {
+	/*
+	 * Fix a bug - if we have a text file with html and built-in image using
+	 * the data:// url. If we press '\' to toggle the view from text to
+	 * html, the image is not displayed correctly. We need to reset
+	 * fdatac->done to force re-parse.
+	 */
+				fdatac->done = 0;
+			}
 		}
 	}
 	return chopped;
@@ -977,7 +988,7 @@ int get_foreground(int rgb)
 static void draw_frame_mark(struct graphics_device *dev, int x, int y, int xw, int yw, long bg, long fg, int broken)
 {
 #ifdef DEBUG
-	if (xw<1||yw<1) internal("zero dimension in draw_frame_mark");
+	if (xw<1||yw<1) internal_error("zero dimension in draw_frame_mark");
 #endif /* #ifdef DEBUG */
 	if (broken == 1){
 		/* Draw between ( 0 and 1/4 ) and ( 3/4 and 1 ) of each
@@ -1061,7 +1072,7 @@ static void draw_picture(struct f_data_c *fdatac, struct g_object_image *goi, in
 #ifdef DEBUG
 	if (goi->cimg->state < 12 || goi->cimg->state >= 16) {
 		fprintf(stderr, "cimg->state=%d\n", cimg->state);
-		internal("Invalid cimg->state in draw_picture");
+		internal_error("Invalid cimg->state in draw_picture");
 	}
 #endif /* #ifdef DEBUG */
 	if (!(cimg->state & 1)) {
@@ -1071,7 +1082,7 @@ static void draw_picture(struct f_data_c *fdatac, struct g_object_image *goi, in
 #ifdef DEBUG
 	else if (!cimg->bmp_used) {
 		fprintf(stderr, "cimg->state=%d\n", cimg->state);
-		internal("Nonexistent bitmap in said cimg->state in draw_picture");
+		internal_error("Nonexistent bitmap in said cimg->state in draw_picture");
 	}
 #endif /* #ifdef DEBUG */
 	restrict_clip_area(dev, &saved, x, y, x + goi->goti.go.xw, y + goi->goti.go.yw);
@@ -1093,7 +1104,7 @@ static void update_bitmap(struct cached_image *cimg)
 #ifdef DEBUG
 	if (cimg->state < 12 || cimg->state >= 16) {
 		fprintf(stderr, "cimg->state=%d\n", cimg->state);
-		internal("Invalid state in update_bitmap");
+		internal_error("Invalid state in update_bitmap");
 	}
 #endif /* #ifdef DEBUG */
 	if (!(cimg->state & 1) && !cimg->strip_optimized && cimg->rows_added)
@@ -1135,7 +1146,7 @@ static void img_draw_image(struct f_data_c *fdatac, struct g_object *goi_, int x
 	} else {
 #ifdef DEBUG
 		if (cimg->state >= 16)
-			internal("Invalid state in img_draw_image: %d", cimg->state);
+			internal_error("Invalid state in img_draw_image: %d", cimg->state);
 #endif
 		update_bitmap(cimg);
 		draw_picture(fdatac, goi, x, y);
@@ -1169,7 +1180,7 @@ static void find_or_make_cached_image(struct g_object_image *image, unsigned cha
 		cimg->background_color = image->background;
 #ifdef DEBUG
 		if (!url)
-			internal ("NULL url as argument of\
+			internal_error("NULL url as argument of\
 find_or_make_cached_image");
 #endif /* #ifdef DEBUG */
 		cimg->scale = scale;
@@ -1206,6 +1217,7 @@ struct g_object_image *insert_image(struct g_part *p, struct image_description *
 	struct g_object_image *image;
 	struct cached_image *cimg;
 	int retval;
+	ssize_t xw, yw;
 
 	image=mem_calloc(sizeof(struct g_object_image));
 	global_goi = image;
@@ -1225,17 +1237,21 @@ struct g_object_image *insert_image(struct g_part *p, struct image_description *
 
 	if (im->autoscale_x && im->autoscale_y) {
 		/* Autoscale requested */
-		image->goti.go.xw = im->autoscale_x;
-		image->goti.go.yw = im->autoscale_y;
+		xw = im->autoscale_x;
+		yw = im->autoscale_y;
+		image->goti.go.xw = (int)xw;
+		image->goti.go.yw = (int)yw;
 		image->xyw_meaning = MEANING_AUTOSCALE;
 	}else{
 		/* Autoscale not requested */
-		image->goti.go.xw = img_scale_h(d_opt->image_scale, im->xsize);
-		image->goti.go.yw = img_scale_v(d_opt->image_scale, im->ysize);
+		xw = img_scale_h(d_opt->image_scale, im->xsize);
+		yw = img_scale_v(d_opt->image_scale, im->ysize);
+		image->goti.go.xw = (int)xw;
+		image->goti.go.yw = (int)yw;
 		image->xyw_meaning = MEANING_DIMS;
 	}
-	if (image->goti.go.xw >= 0 && image->goti.go.yw >= 0) {
-		if (!is_image_size_sane(image->goti.go.xw, image->goti.go.yw)) {
+	if (xw >= 0 && yw >= 0) {
+		if (!is_image_size_sane(xw, yw)) {
 			mem_free(image);
 			return NULL;
 		}
@@ -1273,8 +1289,8 @@ struct g_object_image *insert_image(struct g_part *p, struct image_description *
 next_chunk:
 	retval = img_process_download(image, NULL);
 	if (retval && !(cimg->state & 4)) goto next_chunk;
-	image->goti.go.xw = image->cimg->xww;
-	image->goti.go.yw = image->cimg->yww;
+	image->goti.go.xw = (int)image->cimg->xww;
+	image->goti.go.yw = (int)image->cimg->yww;
 	if (cimg->state == 0 || cimg->state == 8 || (!image->af->rq->ce && image->af->unknown_image_size)) if (image->af->need_reparse != -1) image->af->need_reparse = 1;
 	if (im->insert_flag) add_to_list(current_f_data->images, image);
 	else image->list_entry.prev = image->list_entry.next = NULL;
