@@ -82,7 +82,7 @@ static unsigned long image_size(struct cached_image *cimg)
 #ifdef DEBUG
 		default:
 		fprintf(stderr,"cimg->state=%d\n",cimg->state);
-		internal("Invalid cimg->state in image_size\n");
+		internal_error("Invalid cimg->state in image_size\n");
 		break;
 #endif /* #ifdef DEBUG */
 	}
@@ -128,7 +128,7 @@ my_uintptr_t imgcache_info(int type)
 				n++;
 				break;
 			default:
-				internal("imgcache_info: query %d", type);
+				internal_error("imgcache_info: query %d", type);
 		}
 	}
 	return n;
