@@ -6,7 +6,7 @@
 #include "links.h"
 
 #define OUT_BUF_SIZE		10240
-#define IN_BUF_SIZE		16
+#define IN_BUF_SIZE		64
 
 #define USE_TWIN_MOUSE	1
 #define BRACKETED_PASTE	2
@@ -72,8 +72,8 @@ static void write_ev_queue(void *itrm_)
 {
 	struct itrm *itrm = (struct itrm *)itrm_;
 	int l, to_write;
-	if (!itrm->eqlen) internal("event queue empty");
-	to_write = itrm->eqlen > 128 ? 128 : itrm->eqlen;
+	if (!itrm->eqlen) internal_error("event queue empty");
+	to_write = itrm->eqlen;
 retry:
 	EINTRLOOP(l, (int)write(itrm->sock_out, itrm->ev_queue, to_write));
 	if (l <= 0) {
@@ -137,7 +137,7 @@ static void send_init_sequence(int h, int flags)
 {
 	want_draw();
 	hard_write(h, init_seq, (int)strlen(cast_const_char init_seq));
-#ifndef DOS
+#if !defined(DOS) && !defined(HAIKU)
 	if (flags & USE_TWIN_MOUSE) {
 		hard_write(h, init_seq_tw_mouse, (int)strlen(cast_const_char init_seq_tw_mouse));
 	} else {
@@ -151,7 +151,7 @@ static void send_term_sequence(int h, int flags)
 {
 	want_draw();
 	hard_write(h, term_seq, (int)strlen(cast_const_char term_seq));
-#ifndef DOS
+#if !defined(DOS) && !defined(HAIKU)
 	if (flags & USE_TWIN_MOUSE) {
 		hard_write(h, term_seq_tw_mouse, (int)strlen(cast_const_char term_seq_tw_mouse));
 	} else {
@@ -406,8 +406,8 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 	queue_event(itrm, (unsigned char *)init_string, init_len);
 	itrm->orig_title = get_window_title();
 	set_window_title(cast_uchar "Links");
-	send_init_sequence(std_out, itrm->flags);
 	itrm->mouse_h = handle_mouse(0, mouse_queue_event, itrm);
+	send_init_sequence(std_out, itrm->flags);
 }
 
 static void unblock_itrm_x(void *h)
@@ -426,11 +426,11 @@ int unblock_itrm(int fd)
 	if (setraw(itrm->ctl_in, 0)) return -1;
 	if (itrm->blocked != fd + 1) return -2;
 	itrm->blocked = 0;
-	send_init_sequence(itrm->std_out, itrm->flags);
 	set_handlers(itrm->std_in, in_kbd, NULL, itrm);
 	handle_terminal_resize(resize_terminal, &x, &y);
 	unblock_stdin();
 	itrm->mouse_h = handle_mouse(0, mouse_queue_event, itrm);
+	send_init_sequence(itrm->std_out, itrm->flags);
 	resize_terminal(x, y);
 	return 0;
 }
@@ -635,7 +635,7 @@ static void kbd_timeout(void *itrm_)
 		return;
 	}
 	if (!itrm->qlen) {
-		internal("timeout on empty queue");
+		internal_error("timeout on empty queue");
 		return;
 	}
 	if (itrm->kqueue[0] != 27) {
@@ -992,15 +992,18 @@ static int process_queue(struct itrm *itrm)
 	struct links_event ev = { EV_KBD, -1, 0, 0 };
 	int el = 0;
 	if (!itrm->qlen) goto end;
-	/*{
+#if 0
+	{
 		int i;
-		fprintf(stderr, "queue:");
-		for (i = 0; i < itrm->qlen; i++) {
-			fprintf(stderr, " %d", itrm->kqueue[i]);
-			if (itrm->kqueue[i] >= 32) fprintf(stderr, "[%c]", itrm->kqueue[i]);
-		}
-		fprintf(stderr, ".\n");
-	}*/
+		fprintf(stderr, "queue(%d):", itrm->qlen);
+		for (i = 0; i < itrm->qlen; i++)
+			fprintf(stderr, " %02x", itrm->kqueue[i]);
+		fprintf(stderr, "  \"");
+		for (i = 0; i < itrm->qlen; i++)
+			if (itrm->kqueue[i] >= 32) fprintf(stderr, "%c", itrm->kqueue[i]);
+		fprintf(stderr, "\"\n");
+	}
+#endif
 	if (itrm->kqueue[0] == '\033') {
 		if (itrm->qlen < 2) goto ret;
 		if (itrm->kqueue[1] == '[' || itrm->kqueue[1] == 'O') {
@@ -1126,63 +1129,91 @@ static int process_queue(struct itrm *itrm)
 						xterm_button = itrm->kqueue[el] - ' ';
 						el += 5;
 					} else {
-						int x = 0, y = 0, b = 0, button;
-						unsigned char ch;
+						int x = 0, y = 0, b = 0;
+						int button;
+						unsigned char ch = 0;	/* against warning */
 						if (c == 'M') {
 							/* Legacy mouse protocol: \e[Mbxy whereas b, x and y are raw bytes, offset by 32. */
 							if (itrm->qlen - el < 3) goto ret;
 							b = itrm->kqueue[el++] - ' ';
 							x = itrm->kqueue[el++] - ' ';
 							y = itrm->kqueue[el++] - ' ';
-						} else /* c == '<' */ {
+							/*fprintf(stderr, "b: %d\n", b);*/
+						} else if (c == '<') {
 							/* SGR 1006 mouse extension: \e[<b;x;yM where b, x and y are in decimal, no longer offset by 32,
 							   and the trailing letter is 'm' instead of 'M' for mouse release so that the released button is reported. */
+							int eel;
+							eel = el;
 							while (1) {
 								if (el == itrm->qlen) goto ret;
+								if (el - eel >= 9) goto l1;
 								ch = itrm->kqueue[el++];
 								if (ch == ';') break;
 								if (ch < '0' || ch > '9') goto l1;
-								b = 10 * b + ch - '0';
+								b = 10 * b + (ch - '0');
 							}
+							eel = el;
 							while (1) {
 								if (el == itrm->qlen) goto ret;
+								if (el - eel >= 9) goto l1;
 								ch = itrm->kqueue[el++];
 								if (ch == ';') break;
 								if (ch < '0' || ch > '9') goto l1;
-								x = 10 * x + ch - '0';
+								x = 10 * x + (ch - '0');
 							}
+							eel = el;
 							while (1) {
 								if (el == itrm->qlen) goto ret;
+								if (el - eel >= 9) goto l1;
 								ch = itrm->kqueue[el++];
 								if (ch == 'M' || ch == 'm') break;
 								if (ch < '0' || ch > '9') goto l1;
-								y = 10 * y + ch - '0';
+								y = 10 * y + (ch - '0');
 							}
-							/* Encode a release event the legacy way. */
-							if (ch == 'm') b |= 3;
+						} else {
+							break;
 						}
 						x--;
 						y--;
 						if (x < 0 || y < 0 || b < 0)
 							break;
-						if ((b & ~0x1f) == 0x00) button = B_DOWN;
-						else if ((b & ~0x1f) == 0x20) button = B_DRAG;
-						else {
-							button = -1;
-							goto skip_button;
+
+						/*fprintf(stderr, "b: %d, c %c, ch %c, xterm_button %d\n", b, xterm_button, c, ch);*/
+
+						if (c == 'M' && b == 3) button = B_UP;
+						else if (c == '<' && ch == 'm') button = B_UP;
+						else if ((b & 0x20) == 0x20) button = B_DRAG, b &= ~0x20;
+						else button = B_DOWN;
+
+						if (b == 0) button |= B_LEFT;
+						else if (b == 1) button |= B_MIDDLE;
+						else if (b == 2) button |= B_RIGHT;
+						else if (b == 3 && xterm_button >= 0) button |= xterm_button;
+						else if (b == 0x40) button |= B_WHEELUP;
+						else if (b == 0x41) button |= B_WHEELDOWN;
+						else if (b == 0x42) button |= B_WHEELLEFT;
+						else if (b == 0x43) button |= B_WHEELRIGHT;
+						else if (b == 0x80) button |= B_FOURTH;
+						else if (b == 0x81) button |= B_FIFTH;
+						else if (b == 0x82) button |= B_SIXTH;
+						else break;
+
+						if ((b == 0x80 || b == 0x81 || b == 0x82) && (button & BM_ACT) == B_DOWN && xterm_button == (button & BM_BUTT)) {
+							/* xterm has a bug that it reports down events for both click and release */
+							button &= ~BM_ACT;
+							button |= B_UP;
 						}
-						button |= b & 3;
-						if ((b & 3) == 3) {
-							if (xterm_button == -1)
-								xterm_button = B_LEFT;
-							button = B_UP;
-							button |= xterm_button;
-						}
-						xterm_button = button & BM_BUTT;
-						skip_button:
+
+						if ((button & BM_ACT) == B_DOWN)
+							xterm_button = button & BM_BUTT;
+						if ((button & BM_ACT) == B_UP)
+							xterm_button = -1;
+
 						ev.b = button;
 						ev.x = x;
 						ev.y = y;
+
+						/*fprintf(stderr, "buttons: %02lx\n", ev.b);*/
 					}
 					/*{
 						int a;
@@ -1279,7 +1310,7 @@ do_alt:
 	}
 	l1:
 	if (itrm->qlen < el) {
-		internal("event queue underflow");
+		internal_error("event queue underflow");
 		itrm->qlen = el;
 	}
 	if (ev.x != -1) {
