@@ -54,13 +54,9 @@ void sig_tstp(void *t_)
 	pid_t pid, newpid;
 	EINTRLOOP(pid, getpid());
 #endif
-	if (!F) {
-		block_itrm(1);
-	}
+	if (!F) block_itrm(1);
 #ifdef G
-	else {
-		drv->block(NULL);
-	}
+	else if (drv->block) drv->block(NULL);
 #endif
 #if defined(SIGCONT) && defined(SIGTTOU) && defined(HAVE_GETPID)
 	EINTRLOOP(newpid, fork());
@@ -90,15 +86,12 @@ void sig_tstp(void *t_)
 static void poll_fg(void *t_)
 {
 	struct terminal *t = (struct terminal *)t_;
-	int r;
+	int r = 0;
 	fg_poll_timer = NULL;
-	if (!F) {
-		r = unblock_itrm(1);
+	if (!F) r = unblock_itrm(1);
 #ifdef G
-	} else {
-		r = drv->unblock(NULL);
+	else if (drv->unblock) r = drv->unblock(NULL);
 #endif
-	}
 	if (r == -1) {
 		fg_poll_timer = install_timer(FG_POLL_TIME, poll_fg, t);
 	}
@@ -112,13 +105,10 @@ static void poll_fg(void *t_)
 
 void sig_cont(void *t_)
 {
-	if (!F) {
-		unblock_itrm(1);
+	if (!F) unblock_itrm(1);
 #ifdef G
-	} else {
-		drv->unblock(NULL);
+	else if (drv->unblock) drv->unblock(NULL);
 #endif
-	}
 }
 
 static void handle_basic_signals(struct terminal *term)
@@ -279,9 +269,11 @@ static void end_dump(struct object_request *r, void *p)
 				if (l >= MAXINT) l = MAXINT;
 				w = hard_write(oh, frag->data + dump_pos - frag->offset, (int)l);
 				if (w != l) {
+					unsigned char *msg = strerror_alloc(errno, NULL);
 					detach_object_connection(r, dump_pos);
-					if (w < 0) fprintf(stderr, "Error writing to stdout: %s.\n", strerror(errno));
+					if (w < 0) fprintf(stderr, "Error writing to stdout: %s.\n", msg);
 					else fprintf(stderr, "Can't write to stdout.\n");
+					mem_free(msg);
 					retval = RET_ERROR;
 					goto terminate;
 				}
@@ -294,6 +286,7 @@ static void end_dump(struct object_request *r, void *p)
 	} else if (ce) {
 		struct document_options o;
 		struct f_data_c *fd;
+		int err;
 		fd = create_f_data_c(NULL, NULL);
 		memset(&o, 0, sizeof(struct document_options));
 		o.xp = 0;
@@ -311,13 +304,16 @@ static void end_dump(struct object_request *r, void *p)
 			o.assume_cp = get_commandline_charset();
 		}
 		if (!(fd->f_data = cached_format_html(fd, r, r->url, &o, NULL, 0))) goto term_1;
-		dump_to_file(fd->f_data, oh);
+		if ((err = dump_to_file(fd->f_data, oh))) {
+			fprintf(stderr, "Error writing to stdout: %s.\n", get_err_msg(err, NULL));
+			retval = RET_ERROR;
+		}
 		term_1:
 		reinit_f_data_c(fd);
 		mem_free(fd);
 	}
 	if (r->state != O_OK) {
-		unsigned char *m = get_err_msg(r->stat.state);
+		unsigned char *m = get_err_msg(r->stat.state, NULL);
 		fprintf(stderr, "%s\n", get_english_translation(m));
 		retval = RET_ERROR;
 		goto terminate;
@@ -463,8 +459,9 @@ static void init(void)
 						return;
 					}
 				}
-				spawn_font_thread();
 				init_dither(drv->depth);
+				fontconfig_init();
+				freetype_init();
 			}
 #else
 			fprintf(stderr, "Graphics not enabled when compiling\n");
@@ -565,6 +562,7 @@ static void terminate_all_subsystems(void)
 	end_config();
 	free_strerror_buf();
 	shutdown_trans();
+	GF(freetype_done());
 	GF(free_dither());
 	GF(shutdown_graphics());
 	af_unix_close();
@@ -685,7 +683,7 @@ int main(int argc, char *argv[])
 				fprintf(stderr, "'%s'\n", str);
 				unsigned char *d = puny_decode(str);
 				if (!d || strcmp(cast_const_char d, cast_const_char g_argv[i])) {
-					internal("mismatch - %s", d);
+					internal_error("mismatch - %s", d);
 				}
 				mem_free(str);
 				mem_free(d);
@@ -762,7 +760,7 @@ err:
 			for (i = 0; i < n; i++) {
 				fprintf(stderr, " %u", unistr[i]);
 			}
-			internal("failed");
+			internal_error("failed");
 		}
 		count++;
 		if (!(count % 10000)) {
@@ -775,6 +773,18 @@ err:
 	select_loop(init);
 	terminate_all_subsystems();
 
+#if 0
+	{
+		char msg[65536] = "";
+		char *p = msg;
+		int i = 1;
+		while (p - msg < 60000) {
+			sprintf(p, "some text %d (%ld) ", i++, p - msg);
+			p = strchr(p, 0);
+		}
+		error(msg);
+	}
+#endif
 	check_memory_leaks();
 	return retval;
 }
