@@ -1340,6 +1340,15 @@ static void pm_dispatcher(void *p)
 #endif
 #endif
 #ifdef WIN
+	{
+		unsigned long (WINAPI *f_SetThreadDpiAwarenessContext)(unsigned long ctx);
+		f_SetThreadDpiAwarenessContext = (void *)GetProcAddress(GetModuleHandleA("user32.dll"), "SetThreadDpiAwarenessContext");
+		/*debug("result: %p", f_SetThreadDpiAwarenessContext);*/
+		if (f_SetThreadDpiAwarenessContext) {
+			f_SetThreadDpiAwarenessContext(-3);
+		}
+	}
+
 	pm_cp = get_windows_cp(0);
 
 	hcursor = LoadCursorA(0, (void *)IDC_ARROW);
@@ -2018,7 +2027,7 @@ static void pm_shutdown_driver(void)
 	if (!WinTerminate(hab))
 		error("WinTerminate failed: %lx", WinGetLastError(hab));
 #endif
-	if (pm_bitmap_count) internal("pm_shutdown_driver: %d bitmaps leaked", pm_bitmap_count);
+	if (pm_bitmap_count) internal_error("pm_shutdown_driver: %d bitmaps leaked", pm_bitmap_count);
 }
 
 #ifdef OS2
@@ -2026,7 +2035,8 @@ static void pm_shutdown_driver(void)
 static HPOINTER pmshell_create_icon(void)
 {
 	unsigned char *icon_data, *icon_data_2;
-	int icon_w, icon_h, icon_skip;
+	int icon_w, icon_h;
+	ssize_t icon_skip;
 	int size;
 	int i, j;
 	HBITMAP hbm;
@@ -2035,7 +2045,7 @@ static HPOINTER pmshell_create_icon(void)
 	const int border = 4;
 
 	get_links_icon(&icon_data, &icon_w, &icon_h, &icon_skip, 4);
-	size = icon_h * icon_skip;
+	size = (int)(icon_h * icon_skip);
 	icon_data_2 = pm_alloc(size * 2, 0);
 	for (i = 0; i < icon_h; i++)
 		memcpy(icon_data_2 + i * icon_skip, icon_data + (icon_h - 1 - i) * icon_skip, icon_skip);
@@ -2076,7 +2086,8 @@ static HPOINTER pmshell_create_icon(void)
 static HICON win32_create_icon(int x, int y)
 {
 	unsigned char *icon_data, *zero_data;
-	int icon_w, icon_h, icon_skip;
+	int icon_w, icon_h;
+	ssize_t icon_skip;
 	int left, right, top, bottom;
 	int i, j;
 	HBITMAP hbm_icon;
@@ -2426,7 +2437,7 @@ static void *pm_prepare_strip(struct bitmap *bmp, int top, int lines)
 		return NULL;
 	}
 	if (bmp->data)
-		internal("pm_prepare_strip: bmp->data should not be set here");
+		internal_error("pm_prepare_strip: bmp->data should not be set here");
 	if (-bmp->skip && (unsigned)-bmp->skip * (unsigned)lines / (unsigned)-bmp->skip != (unsigned)lines) goto over;
 	if ((unsigned)-bmp->skip * (unsigned)lines > MAXINT) goto over;
 	bmp->data = pm_alloc(-bmp->skip * lines, PM_ALLOC_MAYFAIL);
@@ -2474,7 +2485,7 @@ static void pm_unregister_bitmap(struct bitmap *bmp)
 #ifdef OS2
 	if ((HBITMAP)bmp->flags != GPI_ERROR) {
 		if (bmp->data)
-			internal("pm_unregister_bitmap: bmp->data should not be set here");
+			internal_error("pm_unregister_bitmap: bmp->data should not be set here");
 		if (!GpiDeleteBitmap((HBITMAP)bmp->flags))
 			error("GpiDeleteBitmap failed: %lx", WinGetLastError(hab));
 	} else
@@ -2877,8 +2888,8 @@ struct graphics_driver pmshell_driver = {
 	pm_scroll,
 	NULL,
 	pm_flush,
-	dummy_block,
-	dummy_unblock,
+	NULL, /* block */
+	NULL, /* unblock */
 	pm_set_palette,
 	NULL, /* get_real_colors */
 	pm_set_window_title,
