@@ -238,7 +238,7 @@ void png_restart(struct cached_image *cimg, unsigned char *data, int length)
 
 #ifdef DEBUG
 	if (!cimg->decoder)
-		internal("decoder NULL in png_restart\n");
+		internal_error("decoder NULL in png_restart\n");
 
 #endif /* #ifdef DEBUG */
 	h = close_stderr();
@@ -269,7 +269,7 @@ void add_png_version(unsigned char **s, int *l)
 #else
 	add_to_str(s, l, cast_uchar PNG_LIBPNG_VER_STRING);
 #endif
-	add_to_str(s, l, cast_uchar ")");
+	add_chr_to_str(s, l, ')');
 }
 
 #endif /* #ifdef G */
