@@ -56,7 +56,7 @@ unsigned long connect_info(int type)
 #endif
 			return list_size(&keepalive_connections);
 		default:
-			internal("connect_info: bad request");
+			internal_error("connect_info: bad request");
 	}
 	return 0;
 }
@@ -65,7 +65,7 @@ static int getpri(struct connection *c)
 {
 	int i;
 	for (i = 0; i < N_PRI; i++) if (c->pri[i]) return i;
-	internal("connection has no owner");
+	internal_error("connection has no owner");
 	return N_PRI;
 }
 
@@ -191,6 +191,7 @@ int get_keepalive_socket(struct connection *c, int *protocol_data)
 	mem_free(k);
 	c->sock1 = cc;
 	if (max_tries == 1) c->tries = -1;
+	c->keepalive = 1;
 	return 0;
 }
 
@@ -218,7 +219,7 @@ static void free_connection_data(struct connection *c)
 		c->pid = 0;
 	}
 	if (!c->running) {
-		internal("connection already suspended");
+		internal_error("connection already suspended");
 	}
 	c->running = 0;
 	if (c->dnsquery) kill_dns_request(&c->dnsquery);
@@ -236,7 +237,7 @@ static void free_connection_data(struct connection *c)
 	}
 	clear_connection_timeout(c);
 	if (--active_connections < 0) {
-		internal("active connections underflow");
+		internal_error("active connections underflow");
 		active_connections = 0;
 	}
 	if (c->state != S_WAIT) {
@@ -246,7 +247,7 @@ static void free_connection_data(struct connection *c)
 				mem_free(h->host);
 				mem_free(h);
 			}
-		} else internal("suspending connection that is not on the list (state %d)", c->state);
+		} else internal_error("suspending connection that is not on the list (state %d)", c->state);
 	}
 }
 
@@ -304,6 +305,9 @@ static void del_connection(struct connection *c)
 	}
 	mem_free(c->url);
 	if (c->prev_url) mem_free(c->prev_url);
+#ifdef HAVE_SSL
+	freeSSL(c->ssl);
+#endif
 	mem_free(c);
 }
 
@@ -313,7 +317,7 @@ void add_keepalive_socket(struct connection *c, uttime timeout, int protocol_dat
 	int rs;
 	free_connection_data(c);
 	if (c->sock1 == -1) {
-		internal("keepalive connection not connected");
+		internal_error("keepalive connection not connected");
 		goto del;
 	}
 	k = mem_alloc(sizeof(struct k_conn));
@@ -334,6 +338,7 @@ void add_keepalive_socket(struct connection *c, uttime timeout, int protocol_dat
 	k->protocol_data = protocol_data;
 #ifdef HAVE_SSL
 	k->ssl = c->ssl;
+	c->ssl = NULL;
 #endif
 	memcpy(&k->last_lookup_state, &c->last_lookup_state, sizeof(struct lookup_state));
 	add_to_list(keepalive_connections, k);
@@ -387,7 +392,7 @@ static void check_keepalive_connections(void)
 		if (!list_empty(keepalive_connections))
 			del_keepalive_socket(list_struct(keepalive_connections.prev, struct k_conn));
 		else
-			internal("keepalive list empty");
+			internal_error("keepalive list empty");
 	if (!list_empty(keepalive_connections))
 		keepalive_timeout = install_timer(KEEPALIVE_CHECK_TIME, keepalive_timer, NULL);
 }
@@ -459,7 +464,7 @@ static int try_to_suspend_connection(struct connection *c, unsigned char *ho)
 	return -1;
 }
 
-static int is_noproxy_url(unsigned char *url)
+int is_noproxy_url(unsigned char *url)
 {
 	unsigned char *host = get_host_name(url);
 	if (!proxies.only_proxies) {
@@ -490,7 +495,7 @@ static void run_connection(struct connection *c)
 	struct h_conn *hc;
 	void (*func)(struct connection *);
 	if (c->running) {
-		internal("connection already running");
+		internal_error("connection already running");
 		return;
 	}
 
@@ -528,6 +533,7 @@ static void run_connection(struct connection *c)
 	}
 	hc->conn++;
 	active_connections++;
+	c->keepalive = 0;
 	c->running = 1;
 	func(c);
 }
@@ -624,23 +630,23 @@ static void check_queue_bugs(void)
 		cc += d->running;
 		if (q < p) {
 			if (!ps) {
-				internal("queue is not sorted");
+				internal_error("queue is not sorted");
 				sort_queue();
 				ps = 1;
 				goto again;
 			} else {
-				internal("queue is not sorted even after sort_queue!");
+				internal_error("queue is not sorted even after sort_queue!");
 				break;
 			}
 		} else p = q;
 		if (d->state < 0) {
-			internal("interrupted connection on queue (conn %s, state %d)", d->url, d->state);
+			internal_error("interrupted connection on queue (conn %s, state %d)", d->url, d->state);
 			ld = ld->prev;
 			abort_connection(d);
 		}
 	}
 	if (cc != active_connections) {
-		internal("bad number of active connections (counted %d, stored %d)", cc, active_connections);
+		internal_error("bad number of active connections (counted %d, stored %d)", cc, active_connections);
 		active_connections = cc;
 	}
 }
@@ -771,7 +777,7 @@ void load_url(unsigned char *url, unsigned char *prev_url, struct status *stat,
 		struct list_head *lst;
 		foreach(struct status, st, lst, c->statuss) {
 			if (st == stat) {
-				internal("status already assigned to '%s'", c->url);
+				internal_error("status already assigned to '%s'", c->url);
 				stat->state = S_INTERNAL;
 				if (stat->end) stat->end(stat, stat->data);
 				goto ret;
@@ -956,7 +962,7 @@ void change_connection(struct status *oldstat, struct status *newstat, int newpr
 	check_queue_bugs();
 	c = oldstat->c;
 	if (--c->pri[oldpri] < 0) {
-		internal("priority counter underflow");
+		internal_error("priority counter underflow");
 		c->pri[oldpri] = 0;
 	}
 	c->pri[newpri]++;
@@ -996,7 +1002,7 @@ void detach_connection(struct status *stat, off_t pos, int want_to_restart, int
 	if (!dont_check_refcount && l < (longlong)memory_cache_size * MAX_CACHED_OBJECT && !want_to_restart) return;
 	n_users = 0;
 	for (i = 0; i < PRI_CANCEL; i++) n_users += c->pri[i];
-	if (!n_users) internal("detaching free connection");
+	if (!n_users) internal_error("detaching free connection");
 	if (n_users != 1) return;
 	shrink_memory(SH_CHECK_QUOTA, 0);
 	detach_cache_entry(c->cache);
@@ -1056,6 +1062,16 @@ void set_connection_timeout(struct connection *c)
 	c->timer = install_timer(get_timeout_value(c), connection_timeout_1, c);
 }
 
+void set_connection_timeout_keepal(struct connection *c)
+{
+	if (c->keepalive && !c->unrestartable) {
+		clear_connection_timeout(c);
+		c->timer = install_timer(timeout_multiple_addresses * 1000, connection_timeout, c);
+		return;
+	}
+	set_connection_timeout(c);
+}
+
 void abort_all_connections(void)
 {
 	while (!list_empty(queue)) {
