@@ -74,7 +74,7 @@ static int can_do_io(int fd, int wr, int sec)
 	int rs;
 
 	if (fd < 0)
-		internal("can_do_io: handle %d", fd);
+		internal_error("can_do_io: handle %d", fd);
 
 #if defined(USE_POLL)
 	{
@@ -176,7 +176,7 @@ unsigned long select_info(int type)
 		case CI_TIMERS:
 			return list_size(&timers);
 		default:
-			internal("select_info_info: bad request");
+			internal_error("select_info_info: bad request");
 	}
 	return 0;
 }
@@ -290,7 +290,7 @@ static void event_callback(int h, short ev, void *data)
 		fatal_exit("ERROR: event_add failed: %s", strerror(errno));
 #endif
 	if (!(ev & EV_READ) == !(ev & EV_WRITE))
-		internal("event_callback: invalid flags %d on handle %d", (int)ev, h);
+		internal_error("event_callback: invalid flags %d on handle %d", (int)ev, h);
 	if (ev & EV_READ) {
 #if defined(HAVE_LIBEV)
 		/* Old versions of libev badly interact with fork and fire
@@ -470,13 +470,13 @@ void add_event_string(unsigned char **s, int *l, struct terminal *term)
 	add_to_str(s, l, cast_uchar "LibEvent");
 #endif
 #ifdef HAVE_EVENT_GET_VERSION
-	add_to_str(s, l, cast_uchar " ");
+	add_chr_to_str(s, l, ' ');
 	{
 #if defined(HAVE_LIBEV)
 				/* old libev report bogus version */
 		if (!casestrcmp(cast_uchar event_get_version(), cast_uchar "EV_VERSION_MAJOR.EV_VERSION_MINOR")) {
 			add_num_to_str(s, l, ev_version_major());
-			add_to_str(s, l, cast_uchar ".");
+			add_chr_to_str(s, l, '.');
 			add_num_to_str(s, l, ev_version_minor());
 		} else
 #endif
@@ -484,15 +484,15 @@ void add_event_string(unsigned char **s, int *l, struct terminal *term)
 	}
 #endif
 	if (!event_enabled) {
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_to_str(s, l, get_text_translation(TEXT_(T_dISABLED), term));
-		add_to_str(s, l, cast_uchar ")");
+		add_chr_to_str(s, l, ')');
 	} else {
 #if defined(HAVE_EVENT_BASE_GET_METHOD)
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_to_str(s, l, cast_uchar event_base_get_method(event_base));
 #elif defined(HAVE_EVENT_GET_METHOD)
-		add_to_str(s, l, cast_uchar " ");
+		add_chr_to_str(s, l, ' ');
 		add_to_str(s, l, cast_uchar event_get_method());
 #endif
 	}
@@ -582,21 +582,21 @@ void kill_timer(struct timer *tm)
 void (*get_handler(int fd, int tp))(void *)
 {
 	if (fd < 0)
-		internal("get_handler: handle %d", fd);
+		internal_error("get_handler: handle %d", fd);
 	if (fd >= w_max)
 		return NULL;
 	switch (tp) {
 		case H_READ:	return threads[fd].read_func;
 		case H_WRITE:	return threads[fd].write_func;
 	}
-	internal("get_handler: bad type %d", tp);
+	internal_error("get_handler: bad type %d", tp);
 	return NULL;
 }
 
 void *get_handler_data(int fd)
 {
 	if (fd < 0)
-		internal("get_handler: handle %d", fd);
+		internal_error("get_handler: handle %d", fd);
 	if (fd >= w_max)
 		return NULL;
 	return threads[fd].data;
@@ -606,7 +606,7 @@ void set_handlers_file_line(int fd, void (*read_func)(void *), void (*write_func
 {
 	if (fd < 0)
 		goto invl;
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(socket)
 	{
 		struct stat st;
 		int rs;
@@ -662,7 +662,7 @@ void set_handlers_file_line(int fd, void (*read_func)(void *), void (*write_func
 	return;
 
 invl:
-	internal("invalid set_handlers call at %s:%d: %d, %p, %p, %p", sh_file, sh_line, fd, read_func, write_func, data);
+	internal_error("invalid set_handlers call at %s:%d: %d, %p, %p, %p", sh_file, sh_line, fd, read_func, write_func, data);
 }
 
 void clear_events(int h, int blocking)
@@ -701,7 +701,7 @@ struct signal_handler {
 static volatile int signal_mask[NUM_SIGNALS];
 static volatile struct signal_handler signal_handlers[NUM_SIGNALS];
 
-pid_t signal_pid;
+static pid_t signal_pid;
 int signal_pipe[2];
 
 SIGNAL_HANDLER static void got_signal(int sig)
@@ -750,7 +750,7 @@ void install_signal_handler(int sig, void (*fn)(void *), void *data, int critica
 	struct sigaction sa = sa_zero;
 	/*debug("install (%d) -> %p,%d", sig, fn, critical);*/
 	if (sig >= NUM_SIGNALS || sig < 0) {
-		internal("bad signal number: %d", sig);
+		internal_error("bad signal number: %d", sig);
 		return;
 	}
 	if (!fn) sa.sa_handler = SIG_IGN;
@@ -779,7 +779,7 @@ void interruptible_signal(int sig, int in)
 	struct sigaction sa = sa_zero;
 	int rs;
 	if (sig >= NUM_SIGNALS || sig < 0) {
-		internal("bad signal number: %d", sig);
+		internal_error("bad signal number: %d", sig);
 		return;
 	}
 	if (!signal_handlers[sig].fn) return;
@@ -836,7 +836,7 @@ static int check_signals(void)
 {
 	int r = 0;
 #ifndef NO_SIGNAL_HANDLERS
-	int i;
+	volatile int i;	/* volatile because of setjmp */
 	for (i = 0; i < NUM_SIGNALS; i++)
 		if (signal_mask[i]) {
 			signal_mask[i] = 0;
@@ -914,8 +914,9 @@ int omp_start(void)
 
 void omp_end(void)
 {
-	if (!disable_openmp || num_threads == 1)
-		unblock_signals();
+	if (disable_openmp || num_threads == 1)
+		return;
+	unblock_signals();
 }
 
 #endif
@@ -988,6 +989,7 @@ void select_loop(void (*init)(void))
 				redraw_all_terminals();
 			}
 			if (terminate_loop) break;
+			test_detach_console();
 			do_event_loop(EVLOOP_ONCE);
 		}
 		nopr();
@@ -995,13 +997,14 @@ void select_loop(void (*init)(void))
 #endif
 
 	while (!terminate_loop) {
-		volatile int n;	/* volatile because of setjmp */
-		int i;
+		volatile int n, i;	/* volatile because of setjmp */
+		int ww_max;
 		struct timeval tv;
 		struct timeval *tm = NULL;
 		check_signals();
 		check_timers();
 		if (!F) redraw_all_terminals();
+		test_detach_console();
 #ifdef OS_BAD_SIGNALS
 	/* Cygwin has buggy signals that sometimes don't interrupt select.
 	   So don't wait indefinitely in it. */
@@ -1037,6 +1040,7 @@ void select_loop(void (*init)(void))
 #ifdef DEBUG_CALLS
 		fprintf(stderr, "select\n");
 #endif
+		ww_max = w_max;
 		if ((n = loop_select(w_max, &x_read, &x_write, NULL, tm)) < 0) {
 #ifdef DEBUG_CALLS
 			fprintf(stderr, "select intr\n");
@@ -1050,13 +1054,16 @@ void select_loop(void (*init)(void))
 		fprintf(stderr, "select done\n");
 #endif
 		check_signals();
-		/*printf("sel: %d\n", n);*/
 		check_timers();
+		/*
+		 * EMX has broken select. It should return the total number of
+		 * bits set, but instead it returns the number of handles for
+		 * which some activity is reported.
+		 * Sometimes it returns unusually high values.
+		 */
 		i = -1;
-		while (n > 0 && ++i < w_max) {
+		while (n > 0 && ++i < ww_max) {
 			int k = 0;
-			/*printf("C %d : %d,%d\n",i,FD_ISSET(i, &w_read),FD_ISSET(i, &w_write));
-			printf("A %d : %d,%d\n",i,FD_ISSET(i, &x_read),FD_ISSET(i, &x_write));*/
 			if (FD_ISSET(i, &x_read)) {
 				if (threads[i].read_func) {
 #ifdef DEBUG_CALLS
