@@ -285,7 +285,7 @@ static void assoc_edit_item(struct dialog_data *dlg, struct list *data, void (*o
 			break;
 
 		default:
-			internal("Unsupported dialog title.\n");
+			internal_error("Unsupported dialog title.\n");
 	}
 
 	d->udata = neww;
@@ -629,7 +629,7 @@ static void ext_edit_item(struct dialog_data *dlg, struct list *data, void (*ok_
 			break;
 
 		default:
-			internal("Unsupported dialog title.\n");
+			internal_error("Unsupported dialog title.\n");
 	}
 
 	d->udata = neww;
@@ -734,7 +734,7 @@ unsigned char *get_prog(struct list_head *l)
 	foreach(struct protocol_program, repl, lrepl, *l) if (repl->system == SYSTEM_ID) return repl->prog;
 	update_prog(l, cast_uchar "", SYSTEM_ID);
 	foreach(struct protocol_program, repl, lrepl, *l) if (repl->system == SYSTEM_ID) return repl->prog;
-	internal("get_prog: program was not added");
+	internal_error("get_prog: program was not added");
 	return cast_uchar "";;
 }
 
@@ -843,6 +843,7 @@ unsigned char *get_compress_by_extension(unsigned char *ext, unsigned char *ext_
 	if (len == 1 && !casecmp(ext, cast_uchar "z", 1)) return cast_uchar "compress";
 	if (len == 2 && !casecmp(ext, cast_uchar "gz", 2)) return cast_uchar "gzip";
 	if (len == 2 && !casecmp(ext, cast_uchar "br", 2)) return cast_uchar "br";
+	if (len == 3 && !casecmp(ext, cast_uchar "zst", 3)) return cast_uchar "zstd";
 	if (len == 3 && !casecmp(ext, cast_uchar "bz2", 3)) return cast_uchar "bzip2";
 	if (len == 4 && !casecmp(ext, cast_uchar "lzma", 4)) return cast_uchar "lzma";
 	if (len == 2 && !casecmp(ext, cast_uchar "xz", 2)) return cast_uchar "lzma2";
@@ -965,6 +966,7 @@ static unsigned char *get_extension_by_content_type(unsigned char *ct)
 		    casestrcmp(x, cast_uchar "gz") &&
 		    casestrcmp(x, cast_uchar "gzip") &&
 		    casestrcmp(x, cast_uchar "br") &&
+		    casestrcmp(x, cast_uchar "zst") &&
 		    casestrcmp(x, cast_uchar "bz2") &&
 		    casestrcmp(x, cast_uchar "bzip2") &&
 		    casestrcmp(x, cast_uchar "lzma") &&
@@ -985,6 +987,8 @@ static unsigned char *get_content_encoding_from_content_type(unsigned char *ct)
 	    !casestrcmp(ct, cast_uchar "application/x-tgz") ||
 	    !casestrcmp(ct, cast_uchar "application/x-gtar")) return cast_uchar "gzip";
 	if (!casestrcmp(ct, cast_uchar "application/x-br")) return cast_uchar "br";
+	if (!casestrcmp(ct, cast_uchar "application/x-zstd") ||
+	    !casestrcmp(ct, cast_uchar "application/zstd")) return cast_uchar "zstd";
 	if (!casestrcmp(ct, cast_uchar "application/x-bzip2") ||
 	    !casestrcmp(ct, cast_uchar "application/x-bzip")) return cast_uchar "bzip2";
 	if (!casestrcmp(ct, cast_uchar "application/x-lzma")) return cast_uchar "lzma";
@@ -1084,6 +1088,8 @@ unsigned char *encoding_2_extension(unsigned char *encoding)
 	    !casestrcmp(encoding, cast_uchar "x-gzip")) return cast_uchar "gz";
 	if (!casestrcmp(encoding, cast_uchar "compress") ||
 	    !casestrcmp(encoding, cast_uchar "x-compress")) return cast_uchar "Z";
+	if (!casestrcmp(encoding, cast_uchar "br")) return cast_uchar "br";
+	if (!casestrcmp(encoding, cast_uchar "zstd")) return cast_uchar "zst";
 	if (!casestrcmp(encoding, cast_uchar "bzip2")) return cast_uchar "bz2";
 	if (!casestrcmp(encoding, cast_uchar "lzma")) return cast_uchar "lzma";
 	if (!casestrcmp(encoding, cast_uchar "lzma2")) return cast_uchar "xz";
